@@ -1,4 +1,4 @@
-// Copyright 2018-2025 the Deno authors. MIT license.
+// Copyright 2018-2026 the Deno authors. MIT license.
 
 use std::collections::HashMap;
 use std::collections::HashSet;
@@ -28,9 +28,8 @@ use deno_npm_installer::graph::NpmCachingStrategy;
 use deno_path_util::resolve_url_or_path;
 use deno_resolver::DenoResolveErrorKind;
 use deno_resolver::display::DisplayTreeNode;
-use deno_semver::npm::NpmPackageNvReference;
 use deno_semver::npm::NpmPackageReqReference;
-use deno_semver::package::PackageNv;
+use deno_semver::package::PackageReq;
 use deno_terminal::colors;
 
 use crate::args::Flags;
@@ -313,6 +312,7 @@ fn add_npm_packages_to_json(
   // ideally deno_graph could handle this, but for now we just modify the json here
   let json = json.as_object_mut().unwrap();
   let modules = json.get_mut("modules").and_then(|m| m.as_array_mut());
+  let mut redirects_to_add = Vec::new();
   if let Some(modules) = modules {
     for module in modules.iter_mut() {
       if matches!(module.get("kind").and_then(|k| k.as_str()), Some("npm")) {
@@ -322,19 +322,40 @@ fn add_npm_packages_to_json(
         let maybe_package = module
           .get("specifier")
           .and_then(|k| k.as_str())
-          .and_then(|specifier| NpmPackageNvReference::from_str(specifier).ok())
-          .and_then(|package_ref| {
+          .and_then(|specifier| {
+            NpmPackageReqReference::from_str(specifier).ok()
+          })
+          .and_then(|package_req_ref| {
             npm_snapshot
-              .resolve_package_from_deno_module(package_ref.nv())
+              .resolve_pkg_from_pkg_req(package_req_ref.req())
               .ok()
+              .map(|pkg| (package_req_ref, pkg))
           });
-        if let Some(pkg) = maybe_package
+        if let Some((pkg_req_ref, pkg)) = maybe_package
           && let Some(module) = module.as_object_mut()
         {
           module.insert(
             "npmPackage".to_string(),
             pkg.id.as_serialized().into_string().into(),
           );
+
+          // for backwards compat, change the specifier
+          // from a req to an nv
+          if let Some(value) = module.get_mut("specifier")
+            && let Some(specifier) = value.as_str()
+          {
+            let new_specifier = format!(
+              "npm:/{}{}",
+              pkg.id.nv,
+              pkg_req_ref
+                .sub_path()
+                .map(|path| format!("/{}", path))
+                .unwrap_or_default()
+            );
+            redirects_to_add
+              .push((specifier.to_string(), new_specifier.clone()));
+            *value = serde_json::Value::String(new_specifier);
+          }
         }
       }
 
@@ -393,13 +414,21 @@ fn add_npm_packages_to_json(
   }
 
   json.insert("npmPackages".to_string(), json_packages.into());
+
+  if let Some(redirects) = json.get_mut("redirects")
+    && let serde_json::Value::Object(redirects) = redirects
+  {
+    for (from, to) in redirects_to_add {
+      redirects.insert(from, to.into());
+    }
+  }
 }
 
 /// Precached information about npm packages that are used in deno info.
 #[derive(Default)]
 struct NpmInfo {
   package_sizes: HashMap<NpmPackageId, u64>,
-  resolved_ids: HashMap<PackageNv, NpmPackageId>,
+  resolved_ids: HashMap<PackageReq, NpmPackageId>,
   packages: HashMap<NpmPackageId, NpmResolutionPackage>,
 }
 
@@ -410,20 +439,17 @@ impl NpmInfo {
     npm_snapshot: &'a NpmResolutionSnapshot,
   ) -> Self {
     let mut info = NpmInfo::default();
-    if graph.npm_packages.is_empty() {
-      return info; // skip going over the modules if there's no npm packages
-    }
 
     for module in graph.modules() {
-      if let Module::Npm(module) = module {
-        // TODO(dsherret): ok to use for now, but we should use the req in the future
-        #[allow(deprecated)]
-        let nv = module.nv_reference.nv();
-        if let Ok(package) = npm_snapshot.resolve_package_from_deno_module(nv) {
-          info.resolved_ids.insert(nv.clone(), package.id.clone());
-          if !info.packages.contains_key(&package.id) {
-            info.fill_package_info(package, npm_resolver, npm_snapshot);
-          }
+      if let Module::Npm(module) = module
+        && let Ok(package) =
+          npm_snapshot.resolve_pkg_from_pkg_req(module.pkg_req_ref.req())
+      {
+        info
+          .resolved_ids
+          .insert(module.pkg_req_ref.req().clone(), package.id.clone());
+        if !info.packages.contains_key(&package.id) {
+          info.fill_package_info(package, npm_resolver, npm_snapshot);
         }
       }
     }
@@ -454,9 +480,9 @@ impl NpmInfo {
 
   pub fn resolve_package(
     &self,
-    nv: &PackageNv,
+    req: &PackageReq,
   ) -> Option<&NpmResolutionPackage> {
-    let id = self.resolved_ids.get(nv)?;
+    let id = self.resolved_ids.get(req)?;
     self.packages.get(id)
   }
 }
@@ -602,10 +628,7 @@ impl<'a> GraphDisplayContext<'a> {
 
     let package_or_specifier = match module.npm() {
       Some(npm) => {
-        // TODO(dsherret): ok to use for now, but we should use the req in the future
-        #[allow(deprecated)]
-        let nv = npm.nv_reference.nv();
-        match self.npm_info.resolve_package(nv) {
+        match self.npm_info.resolve_package(npm.pkg_req_ref.req()) {
           Some(package) => Package(Box::new(package.clone())),
           None => Specifier(module.specifier().clone()), // should never happen
         }
@@ -616,18 +639,22 @@ impl<'a> GraphDisplayContext<'a> {
       Package(package) => package.id.as_serialized().into_string(),
       Specifier(specifier) => specifier.to_string(),
     });
+    let header_text = match &package_or_specifier {
+      Package(package) => format!("npm:/{}", package.id.as_serialized()),
+      Specifier(specifier) => specifier.to_string(),
+    };
     let header_text = if was_seen {
       let specifier_str = if type_dep {
-        colors::italic_gray(module.specifier()).to_string()
+        colors::italic_gray(header_text).to_string()
       } else {
-        colors::gray(module.specifier()).to_string()
+        colors::gray(header_text).to_string()
       };
       format!("{} {}", specifier_str, colors::gray("*"))
     } else {
       let header_text = if type_dep {
-        colors::italic(module.specifier()).to_string()
+        colors::italic(header_text).to_string()
       } else {
-        module.specifier().to_string()
+        header_text
       };
       let maybe_size = match &package_or_specifier {
         Package(package) => {
@@ -743,6 +770,11 @@ impl<'a> GraphDisplayContext<'a> {
       ModuleErrorKind::UnsupportedImportAttributeType { .. } => {
         self.build_error_msg(specifier, "(unsupported import attribute)")
       }
+      ModuleErrorKind::UnsupportedModuleTypeForSourcePhaseImport { .. } => self
+        .build_error_msg(
+          specifier,
+          "(unsupported module type for source phase import)",
+        ),
       ModuleErrorKind::UnsupportedMediaType { .. } => {
         self.build_error_msg(specifier, "(unsupported)")
       }
