@@ -73,7 +73,7 @@ async fn resolve_local_bins(
 
 fn run_js_file(
   main_module: &deno_core::url::Url,
-  permission_args: &[String],
+  deno_args: &[String],
   argv: &[String],
   npm_process_state: Option<String>,
   npm: bool,
@@ -85,7 +85,7 @@ fn run_js_file(
     .context("Failed to get current executable path")?;
 
   let mut args: Vec<std::ffi::OsString> = vec!["run".into()];
-  args.extend(permission_args.iter().map(|s| s.into()));
+  args.extend(deno_args.iter().map(|s| s.into()));
   args.push(main_module.as_str().into());
   args.extend(argv.iter().map(|s| s.into()));
 
@@ -125,15 +125,101 @@ fn run_js_file(
   }
 }
 
+fn get_npm_process_state(npm_resolver: &CliNpmResolver) -> Option<String> {
+  match npm_resolver {
+    deno_resolver::npm::NpmResolver::Managed(managed) => Some(
+      deno_npm_installer::process_state::NpmProcessState::new_managed(
+        managed.resolution().serialized_valid_snapshot(),
+        managed.root_node_modules_path(),
+      )
+      .as_serialized(),
+    ),
+    deno_resolver::npm::NpmResolver::Byonm(_) => None,
+  }
+}
+
+fn run_bin_value(
+  factory: &CliFactory,
+  flags: &Flags,
+  bin_value: BinValue,
+  npm_process_state: Option<String>,
+  unstable_args: &[String],
+) -> Result<i32, AnyError> {
+  match bin_value {
+    BinValue::JsFile(path_buf) => {
+      let path = deno_path_util::url_from_file_path(path_buf.as_ref())?;
+      let mut deno_args = flags.to_permission_args();
+      deno_args.extend(unstable_args.iter().cloned());
+
+      run_js_file(&path, &deno_args, &flags.argv, npm_process_state, true)
+    }
+    BinValue::Executable(mut path_buf) => {
+      if cfg!(windows) && path_buf.extension().is_none() {
+        // prefer cmd shim over sh
+        path_buf.set_extension("cmd");
+        if !path_buf.exists() {
+          //  just fall back to original path
+          path_buf.set_extension("");
+        }
+      }
+      let permissions = factory.root_permissions_container()?;
+      permissions.check_run(
+        &deno_runtime::deno_permissions::RunQueryDescriptor::Path(
+          PathQueryDescriptor::new(
+            &factory.sys(),
+            std::borrow::Cow::Borrowed(path_buf.as_ref()),
+          )?,
+        ),
+        "entrypoint",
+      )?;
+      let mut child = std::process::Command::new(path_buf)
+        .args(&flags.argv)
+        .spawn()
+        .context("Failed to spawn command")?;
+      let status = child.wait()?;
+      Ok(status.code().unwrap_or(1))
+    }
+  }
+}
+
+/// Try to find a bin value from a map of bins, with fallbacks for scoped package names
+/// and single-bin packages.
+fn find_bin_value(
+  bins: &BTreeMap<String, BinValue>,
+  bin_name: &str,
+) -> Option<BinValue> {
+  bins
+    .get(bin_name)
+    .or_else(|| {
+      // Try the package name without scope as fallback
+      if bin_name.starts_with('@') && bin_name.contains('/') {
+        bin_name
+          .split('/')
+          .next_back()
+          .and_then(|name| bins.get(name))
+      } else {
+        None
+      }
+    })
+    .or_else(|| {
+      // If there's only one bin, use it
+      if bins.len() == 1 {
+        bins.values().next()
+      } else {
+        None
+      }
+    })
+    .cloned()
+}
+
 async fn maybe_run_local_npm_bin(
   factory: &CliFactory,
   flags: &Flags,
   node_resolver: &CliNodeResolver,
   npm_resolver: &CliNpmResolver,
   command: &str,
+  unstable_args: &[String],
 ) -> Result<Option<i32>, AnyError> {
-  let permissions = factory.root_permissions_container()?;
-
   let mut bins =
     resolve_local_bins(node_resolver, npm_resolver, factory).await?;
   let bin_value = if let Some(bin_value) = bins.remove(command) {
@@ -151,56 +237,9 @@ async fn maybe_run_local_npm_bin(
     return Ok(None);
   };
 
-  let npm_process_state = match npm_resolver {
-    deno_resolver::npm::NpmResolver::Managed(managed) => Some(
-      deno_npm_installer::process_state::NpmProcessState::new_managed(
-        managed.resolution().serialized_valid_snapshot(),
-        managed.root_node_modules_path(),
-      )
-      .as_serialized(),
-    ),
-    deno_resolver::npm::NpmResolver::Byonm(_) => None,
-  };
-
-  match bin_value {
-    BinValue::JsFile(path_buf) => {
-      let path = deno_path_util::url_from_file_path(path_buf.as_ref())?;
-      let permission_args = flags.to_permission_args();
-      run_js_file(
-        &path,
-        &permission_args,
-        &flags.argv,
-        npm_process_state,
-        true,
-      )
-      .map(Some)
-    }
-    BinValue::Executable(mut path_buf) => {
-      if cfg!(windows) && path_buf.extension().is_none() {
-        // prefer cmd shim over sh
-        path_buf.set_extension("cmd");
-        if !path_buf.exists() {
-          //  just fall back to original path
-          path_buf.set_extension("");
-        }
-      }
-      permissions.check_run(
-        &deno_runtime::deno_permissions::RunQueryDescriptor::Path(
-          PathQueryDescriptor::new(
-            &factory.sys(),
-            std::borrow::Cow::Borrowed(path_buf.as_ref()),
-          )?,
-        ),
-        "entrypoint",
-      )?;
-      let mut child = std::process::Command::new(path_buf)
-        .args(&flags.argv)
-        .spawn()
-        .context("Failed to spawn command")?;
-      let status = child.wait()?;
-      Ok(Some(status.code().unwrap_or(1)))
-    }
-  }
+  let npm_process_state = get_npm_process_state(npm_resolver);
+  run_bin_value(factory, flags, bin_value, npm_process_state, unstable_args)
+    .map(Some)
 }
 
 enum XTempDir {
@@ -333,6 +372,8 @@ pub async fn run(flags: Arc<Flags>, x_flags: XFlags) -> Result<i32, AnyError> {
     }
   };
   let factory = CliFactory::from_flags(flags.clone());
+  let cli_options = factory.cli_options()?;
+  let unstable_args = cli_options.unstable_args();
   let npm_resolver = factory.npm_resolver().await?;
   let node_resolver = factory.node_resolver().await?;
   let result = maybe_run_local_npm_bin(
@@ -341,14 +382,13 @@ pub async fn run(flags: Arc<Flags>, x_flags: XFlags) -> Result<i32, AnyError> {
     node_resolver,
     npm_resolver,
     &command_flags.command,
+    &unstable_args,
   )
   .await?;
   if let Some(exit_code) = result {
     return Ok(exit_code);
   }
 
-  let cli_options = factory.cli_options()?;
-
   let is_file_like = command_flags.command.starts_with('.')
     || command_flags.command.starts_with('/')
     || command_flags.command.starts_with('~')
@@ -386,6 +426,36 @@ pub async fn run(flags: Arc<Flags>, x_flags: XFlags) -> Result<i32, AnyError> {
   let reload = matches!(cache_setting, CacheSetting::ReloadAll);
   match thing_to_run {
     ReqRefOrUrl::Npm(npm_package_req_reference) => {
+      // First try to resolve from the local project
+      if let Ok(package_folder) = npm_resolver
+        .resolve_pkg_folder_from_deno_module_req(
+          npm_package_req_reference.req(),
+          &deno_path_util::url_from_directory_path(cli_options.initial_cwd())
+            .unwrap(),
+        )
+      {
+        let bin_name =
+          if let Some(sub_path) = npm_package_req_reference.sub_path() {
+            sub_path
+          } else {
+            npm_package_req_reference.req().name.as_str()
+          };
+
+        let bins = node_resolver
+          .resolve_npm_binary_commands_for_package(&package_folder)?;
+        if let Some(bin_value) = find_bin_value(&bins, bin_name) {
+          let npm_process_state = get_npm_process_state(npm_resolver);
+          return run_bin_value(
+            &factory,
+            &flags,
+            bin_value,
+            npm_process_state,
+            &unstable_args,
+          );
+        }
+      }
+
+      // Fall back to autoinstall
       let (managed_flags, managed_factory) = autoinstall_package(
         ReqRef::Npm(&npm_package_req_reference),
         &flags,
@@ -415,6 +485,7 @@ pub async fn run(flags: Arc<Flags>, x_flags: XFlags) -> Result<i32, AnyError> {
         runner_node_resolver,
         runner_npm_resolver,
         bin_name,
+        &unstable_args,
       )
       .await?;
       if let Some(exit_code) = res {
@@ -440,6 +511,7 @@ pub async fn run(flags: Arc<Flags>, x_flags: XFlags) -> Result<i32, AnyError> {
             runner_node_resolver,
             runner_npm_resolver,
             fallback_name.as_ref(),
+            &unstable_args,
           )
           .await?
         {
@@ -468,16 +540,7 @@ pub async fn run(flags: Arc<Flags>, x_flags: XFlags) -> Result<i32, AnyError> {
       .await?;
 
       let npm_resolver = new_factory.npm_resolver().await?;
-      let npm_process_state = match npm_resolver {
-        deno_resolver::npm::NpmResolver::Managed(managed) => Some(
-          deno_npm_installer::process_state::NpmProcessState::new_managed(
-            managed.resolution().serialized_valid_snapshot(),
-            managed.root_node_modules_path(),
-          )
-          .as_serialized(),
-        ),
-        deno_resolver::npm::NpmResolver::Byonm(_) => None,
-      };
+      let npm_process_state = get_npm_process_state(npm_resolver);
 
       let permission_args = flags.to_permission_args();
       let url =
