@@ -1,4 +1,4 @@
-// Copyright 2018-2025 the Deno authors. MIT license.
+// Copyright 2018-2026 the Deno authors. MIT license.
 
 use std::borrow::Cow;
 use std::cell::RefCell;
@@ -17,6 +17,7 @@ use deno_core::AsyncRefCell;
 use deno_core::AsyncResult;
 use deno_core::CancelHandle;
 use deno_core::CancelTryFuture;
+use deno_core::FromV8;
 use deno_core::OpState;
 use deno_core::RcRef;
 use deno_core::Resource;
@@ -26,6 +27,7 @@ use deno_core::op2;
 use deno_core::v8;
 use deno_error::JsErrorBox;
 use deno_permissions::OpenAccessKind;
+use deno_permissions::PermissionsContainer;
 use deno_tls::ServerConfigProvider;
 use deno_tls::SocketUse;
 use deno_tls::TlsClientConfigOptions;
@@ -43,13 +45,11 @@ use deno_tls::rustls::pki_types::ServerName;
 pub use rustls_tokio_stream::TlsStream;
 pub use rustls_tokio_stream::TlsStreamRead;
 pub use rustls_tokio_stream::TlsStreamWrite;
-use serde::Deserialize;
 use tokio::io::AsyncReadExt;
 use tokio::io::AsyncWriteExt;
 use tokio::net::TcpStream;
 
 use crate::DefaultTlsOptions;
-use crate::NetPermissions;
 use crate::UnsafelyIgnoreCertificateErrors;
 use crate::io::TcpStreamResource;
 use crate::ops::IpAddr;
@@ -226,8 +226,7 @@ impl Resource for TlsStreamResource {
   }
 }
 
-#[derive(Deserialize)]
-#[serde(rename_all = "camelCase")]
+#[derive(FromV8)]
 pub struct ConnectTlsArgs {
   cert_file: Option<String>,
   ca_certs: Vec<String>,
@@ -236,8 +235,7 @@ pub struct ConnectTlsArgs {
   unsafely_disable_hostname_verification: Option<bool>,
 }
 
-#[derive(Deserialize)]
-#[serde(rename_all = "camelCase")]
+#[derive(FromV8)]
 pub struct StartTlsArgs {
   rid: ResourceId,
   ca_certs: Vec<String>,
@@ -280,7 +278,7 @@ pub fn op_tls_cert_resolver_create<'s>(
   v8::Array::new_with_elements(scope, &[resolver.into(), lookup.into()])
 }
 
-#[op2(async)]
+#[op2]
 #[string]
 pub async fn op_tls_cert_resolver_poll(
   #[cppgc] lookup: &TlsKeyLookup,
@@ -311,15 +309,11 @@ pub fn op_tls_cert_resolver_resolve_error(
 }
 
 #[op2(stack_trace)]
-#[serde]
-pub fn op_tls_start<NP>(
+pub fn op_tls_start(
   state: Rc<RefCell<OpState>>,
-  #[serde] args: StartTlsArgs,
+  #[scoped] args: StartTlsArgs,
   #[cppgc] key_pair: Option<&TlsKeysHolder>,
-) -> Result<(ResourceId, IpAddr, IpAddr), NetError>
-where
-  NP: NetPermissions + 'static,
-{
+) -> Result<(ResourceId, IpAddr, IpAddr), NetError> {
   let rid = args.rid;
   let reject_unauthorized = args.reject_unauthorized.unwrap_or(true);
   let hostname = match &*args.hostname {
@@ -404,17 +398,13 @@ where
   Ok((rid, IpAddr::from(local_addr), IpAddr::from(remote_addr)))
 }
 
-#[op2(async, stack_trace)]
-#[serde]
-pub async fn op_net_connect_tls<NP>(
+#[op2(stack_trace)]
+pub async fn op_net_connect_tls(
   state: Rc<RefCell<OpState>>,
-  #[serde] addr: IpAddr,
-  #[serde] args: ConnectTlsArgs,
+  #[scoped] addr: IpAddr,
+  #[scoped] args: ConnectTlsArgs,
   #[cppgc] key_pair: &TlsKeysHolder,
-) -> Result<(ResourceId, IpAddr, IpAddr), NetError>
-where
-  NP: NetPermissions + 'static,
-{
+) -> Result<(ResourceId, IpAddr, IpAddr), NetError> {
   let cert_file = args.cert_file.as_deref();
   let unsafely_ignore_certificate_errors = state
     .borrow()
@@ -425,7 +415,7 @@ where
 
   let cert_file = {
     let mut s = state.borrow_mut();
-    let permissions = s.borrow_mut::<NP>();
+    let permissions = s.borrow_mut::<PermissionsContainer>();
     permissions
       .check_net(&(&addr.hostname, Some(addr.port)), "Deno.connectTls()")
       .map_err(NetError::Permission)?;
@@ -435,7 +425,7 @@ where
           .check_open(
             Cow::Borrowed(Path::new(path)),
             OpenAccessKind::ReadNoFollow,
-            "Deno.connectTls()",
+            Some("Deno.connectTls()"),
           )
           .map_err(NetError::Permission)?,
       )
@@ -507,33 +497,28 @@ where
   Ok((rid, IpAddr::from(local_addr), IpAddr::from(remote_addr)))
 }
 
-#[derive(Deserialize)]
-#[serde(rename_all = "camelCase")]
+#[derive(FromV8)]
 pub struct ListenTlsArgs {
   alpn_protocols: Option<Vec<String>>,
   reuse_port: bool,
-  #[serde(default)]
+  #[from_v8(default)]
   load_balanced: bool,
   tcp_backlog: i32,
 }
 
 #[op2(stack_trace)]
-#[serde]
-pub fn op_net_listen_tls<NP>(
+pub fn op_net_listen_tls(
   state: &mut OpState,
-  #[serde] addr: IpAddr,
-  #[serde] args: ListenTlsArgs,
+  #[scoped] addr: IpAddr,
+  #[scoped] args: ListenTlsArgs,
   #[cppgc] keys: &TlsKeysHolder,
-) -> Result<(ResourceId, IpAddr), NetError>
-where
-  NP: NetPermissions + 'static,
-{
+) -> Result<(ResourceId, IpAddr), NetError> {
   if args.reuse_port {
     super::check_unstable(state, "Deno.listenTls({ reusePort: true })");
   }
 
   {
-    let permissions = state.borrow_mut::<NP>();
+    let permissions = state.borrow_mut::<PermissionsContainer>();
     permissions
       .check_net(&(&addr.hostname, Some(addr.port)), "Deno.listenTls()")
       .map_err(NetError::Permission)?;
@@ -582,8 +567,7 @@ where
   Ok((rid, IpAddr::from(local_addr)))
 }
 
-#[op2(async)]
-#[serde]
+#[op2]
 pub async fn op_net_accept_tls(
   state: Rc<RefCell<OpState>>,
   #[smi] rid: ResourceId,
@@ -619,7 +603,7 @@ pub async fn op_net_accept_tls(
   Ok((rid, IpAddr::from(local_addr), IpAddr::from(remote_addr)))
 }
 
-#[op2(async)]
+#[op2]
 #[serde]
 pub async fn op_tls_handshake(
   state: Rc<RefCell<OpState>>,
