@@ -1,4 +1,4 @@
-// Copyright 2018-2025 the Deno authors. MIT license.
+// Copyright 2018-2026 the Deno authors. MIT license.
 // Copyright Joyent, Inc. and other Node contributors.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
@@ -30,6 +30,7 @@ import {
   validateFunction,
   validateNumber,
   validateOneOf,
+  validatePort,
   validateString,
 } from "ext:deno_node/internal/validators.mjs";
 import { isIP } from "ext:deno_node/internal/net.ts";
@@ -80,6 +81,8 @@ import {
   dnsException,
   ERR_INVALID_ARG_TYPE,
   ERR_INVALID_ARG_VALUE,
+  ERR_MISSING_ARGS,
+  handleDnsError,
 } from "ext:deno_node/internal/errors.ts";
 import {
   AI_ADDRCONFIG as ADDRCONFIG,
@@ -89,9 +92,13 @@ import {
 import cares, {
   type ChannelWrapQuery,
   GetAddrInfoReqWrap,
+  GetNameInfoReqWrap,
   QueryReqWrap,
 } from "ext:deno_node/internal_binding/cares_wrap.ts";
 import { domainToASCII } from "ext:deno_node/internal/idna.ts";
+import { primordials } from "ext:core/mod.js";
+
+const { ObjectDefineProperty } = primordials;
 
 function onlookup(
   this: GetAddrInfoReqWrap,
@@ -227,8 +234,19 @@ export function lookup(
     }
 
     if (options?.family != null) {
-      validateOneOf(options.family, "options.family", validFamilies);
-      family = options.family;
+      // Accept both numeric (0, 4, 6) and string ('IPv4', 'IPv6') family values
+      // to match Node.js behavior
+      switch (options.family) {
+        case "IPv4":
+          family = 4;
+          break;
+        case "IPv6":
+          family = 6;
+          break;
+        default:
+          validateOneOf(options.family, "options.family", validFamilies);
+          family = options.family;
+      }
     }
 
     if (options?.all != null) {
@@ -305,6 +323,60 @@ Object.defineProperty(lookup, customPromisifyArgs, {
   enumerable: false,
 });
 
+function onlookupservice(
+  this: GetNameInfoReqWrap,
+  err: Error | null,
+  hostname?: string,
+  service?: string,
+) {
+  if (err) {
+    return this.callback!(handleDnsError(err, "getnameinfo", this.address));
+  }
+
+  this.callback!(err, hostname, service);
+}
+
+export function lookupService(
+  address: string,
+  port: number,
+  callback: (
+    err: ErrnoException | null,
+    hostname?: string,
+    service?: string,
+  ) => void,
+): GetNameInfoReqWrap {
+  if (arguments.length !== 3) {
+    throw new ERR_MISSING_ARGS("address", "port", "callback");
+  }
+
+  if (isIP(address) === 0) {
+    throw new ERR_INVALID_ARG_VALUE("address", address);
+  }
+
+  port = validatePort(port);
+
+  validateFunction(callback, "callback");
+
+  const req = new GetNameInfoReqWrap();
+  req.callback = callback;
+  req.address = address;
+  req.port = port;
+  req.oncomplete = onlookupservice;
+
+  const errCode = cares.getnameinfo(req, address, port);
+  if (errCode) {
+    throw dnsException(errCode, "getnameinfo", address);
+  }
+
+  return req;
+}
+
+ObjectDefineProperty(lookupService, customPromisifyArgs, {
+  __proto__: null,
+  value: ["hostname", "service"],
+  enumerable: false,
+});
+
 function onresolve(
   this: QueryReqWrap,
   err: number,
@@ -986,6 +1058,7 @@ export default {
   ALL,
   V4MAPPED,
   lookup,
+  lookupService,
   getServers,
   resolveAny,
   resolve4,
