@@ -1,4 +1,4 @@
-// Copyright 2018-2025 the Deno authors. MIT license.
+// Copyright 2018-2026 the Deno authors. MIT license.
 
 // This module implements 'child_process' module of Node.JS API.
 // ref: https://nodejs.org/api/child_process.html
@@ -9,10 +9,14 @@
 import { core, internals } from "ext:core/mod.js";
 import {
   op_node_in_npm_package,
-  op_node_ipc_read,
+  op_node_ipc_buffer_constructor,
+  op_node_ipc_read_advanced,
+  op_node_ipc_read_json,
   op_node_ipc_ref,
   op_node_ipc_unref,
-  op_node_ipc_write,
+  op_node_ipc_write_advanced,
+  op_node_ipc_write_json,
+  op_node_translate_cli_args,
 } from "ext:core/ops";
 import {
   ArrayIsArray,
@@ -23,10 +27,11 @@ import {
   ArrayPrototypeSort,
   ArrayPrototypeUnshift,
   ObjectHasOwn,
+  StringPrototypeIncludes,
   StringPrototypeStartsWith,
   StringPrototypeToUpperCase,
 } from "ext:deno_node/internal/primordials.mjs";
-import { assert } from "ext:deno_node/_util/asserts.ts";
+import assert from "node:assert";
 import { EventEmitter } from "node:events";
 import { os } from "ext:deno_node/internal_binding/constants.ts";
 import { notImplemented } from "ext:deno_node/_utils.ts";
@@ -37,10 +42,13 @@ import {
   AbortError,
   ERR_INVALID_ARG_TYPE,
   ERR_INVALID_ARG_VALUE,
+  ERR_INVALID_SYNC_FORK_INPUT,
   ERR_IPC_CHANNEL_CLOSED,
+  ERR_IPC_SYNC_FORK,
   ERR_UNKNOWN_SIGNAL,
 } from "ext:deno_node/internal/errors.ts";
 import { Buffer } from "node:buffer";
+import { FastBuffer } from "ext:deno_node/internal/buffer.mjs";
 import { errnoException } from "ext:deno_node/internal/errors.ts";
 import { ErrnoException } from "ext:deno_node/_global.d.ts";
 import { codeMap } from "ext:deno_node/internal_binding/uv.ts";
@@ -48,6 +56,7 @@ import {
   isInt32,
   validateBoolean,
   validateObject,
+  validateOneOf,
   validateString,
 } from "ext:deno_node/internal/validators.mjs";
 import { kEmptyObject } from "ext:deno_node/internal/util.mjs";
@@ -62,6 +71,7 @@ import {
   kInputOption,
   kIpc,
   kNeedsNpmProcessState,
+  kSerialization,
 } from "ext:deno_process/40_process.js";
 
 export function mapValues<T, O>(
@@ -193,12 +203,12 @@ export class ChildProcess extends EventEmitter {
   /**
    * Command line arguments given to this child process.
    */
-  spawnargs: string[];
+  spawnargs: string[] = [];
 
   /**
    * The executable file name of this child process.
    */
-  spawnfile: string;
+  spawnfile: string = "";
 
   /**
    * This property represents the child process's stdin.
@@ -232,22 +242,92 @@ export class ChildProcess extends EventEmitter {
   [kClosesReceived] = 0;
   [kCanDisconnect] = false;
 
-  constructor(
-    command: string,
-    args?: string[],
-    options?: ChildProcessOptions,
-  ) {
+  constructor() {
     super();
+  }
 
+  /**
+   * Internal spawn method used by Node.js internals.
+   * This is called after creating a ChildProcess instance.
+   */
+  spawn(options: {
+    file?: string;
+    args?: string[];
+    cwd?: string;
+    stdio?: Array<NodeStdio | number | Stream | null | undefined> | NodeStdio;
+    envPairs?: string[];
+    windowsVerbatimArguments?: boolean;
+    detached?: boolean;
+    signal?: AbortSignal;
+    serialization?: "json" | "advanced";
+    // deno-lint-ignore no-explicit-any
+    [key: string]: any;
+  }): void {
+    // Validate options
+    if (options == null || typeof options !== "object") {
+      throw new ERR_INVALID_ARG_TYPE("options", "object", options);
+    }
+
+    // Validate envPairs before file (Node.js validation order)
+    const { envPairs } = options;
+    if (envPairs !== undefined && !ArrayIsArray(envPairs)) {
+      throw new ERR_INVALID_ARG_TYPE("options.envPairs", "Array", envPairs);
+    }
+
+    // Validate args
+    const { args } = options;
+    if (args !== undefined && !ArrayIsArray(args)) {
+      throw new ERR_INVALID_ARG_TYPE("options.args", "Array", args);
+    }
+
+    // Validate file
+    const { file } = options;
+    if (file == null || typeof file !== "string") {
+      throw new ERR_INVALID_ARG_TYPE("options.file", "string", file);
+    }
+
+    this.#spawnInternal(file, args || [], options);
+  }
+
+  /**
+   * Internal method that performs the actual spawning.
+   */
+  #spawnInternal(
+    command: string,
+    args: string[],
+    options: {
+      cwd?: string;
+      stdio?: Array<NodeStdio | number | Stream | null | undefined> | NodeStdio;
+      envPairs?: string[];
+      windowsVerbatimArguments?: boolean;
+      detached?: boolean;
+      signal?: AbortSignal;
+      serialization?: "json" | "advanced";
+      // deno-lint-ignore no-explicit-any
+      [key: string]: any;
+    },
+  ): void {
     const {
-      env = {},
       stdio = ["pipe", "pipe", "pipe"],
       cwd,
-      shell = false,
       signal,
       windowsVerbatimArguments = false,
       detached,
-    } = options || {};
+      envPairs,
+    } = options;
+
+    // Convert envPairs array to env object
+    const env: Record<string, string> = {};
+    if (envPairs) {
+      for (const pair of envPairs) {
+        const idx = pair.indexOf("=");
+        if (idx !== -1) {
+          env[pair.substring(0, idx)] = pair.substring(idx + 1);
+        }
+      }
+    }
+
+    const serialization = options.serialization || "json";
     const normalizedStdio = normalizeStdioOption(stdio);
     const [
       stdin = "pipe",
@@ -255,12 +335,15 @@ export class ChildProcess extends EventEmitter {
       stderr = "pipe",
       ...extraStdio
     ] = normalizedStdio;
+
+    // buildCommand handles Node.js to Deno CLI arg translation when spawning Deno
+    // Note: args[0] is argv0 (prepended by normalizeSpawnArguments), so we skip it
     const [cmd, cmdArgs, includeNpmProcessState] = buildCommand(
       command,
-      args || [],
-      shell,
+      args.slice(1),
       env,
     );
+
     this.spawnfile = cmd;
     this.spawnargs = [cmd, ...cmdArgs];
 
@@ -275,23 +358,21 @@ export class ChildProcess extends EventEmitter {
       extraStdioNormalized.push(toDenoStdio(extraStdio[i]));
     }
 
-    const stringEnv = mapValues(env, (value) => value.toString());
     try {
       this.#process = new Deno.Command(cmd, {
         args: cmdArgs,
         clearEnv: true,
         cwd,
-        env: stringEnv,
+        env,
         stdin: toDenoStdio(stdin),
         stdout: toDenoStdio(stdout),
         stderr: toDenoStdio(stderr),
         windowsRawArguments: windowsVerbatimArguments,
         detached,
+        [kSerialization]: serialization,
         [kIpc]: ipc, // internal
         [kExtraStdio]: extraStdioNormalized,
-        [kNeedsNpmProcessState]:
-          // deno-lint-ignore no-explicit-any
-          (options ?? {} as any)[kNeedsNpmProcessState] ||
+        [kNeedsNpmProcessState]: options[kNeedsNpmProcessState] ||
           includeNpmProcessState,
       }).spawn();
       this.pid = this.#process.pid;
@@ -386,7 +467,7 @@ export class ChildProcess extends EventEmitter {
 
       const pipeRid = internals.getIpcPipeRid(this.#process);
       if (typeof pipeRid == "number") {
-        setupChannel(this, pipeRid);
+        setupChannel(this, pipeRid, serialization);
         this[kClosesNeeded]++;
         this.on("disconnect", () => {
           maybeClose(this);
@@ -410,7 +491,8 @@ export class ChildProcess extends EventEmitter {
     } catch (err) {
       let e = err;
       if (e instanceof Deno.errors.NotFound) {
-        e = _createSpawnSyncError("ENOENT", command, args);
+        // args.slice(1) to exclude argv0 (prepended by normalizeSpawnArguments)
+        e = _createSpawnError("ENOENT", command, args.slice(1));
       }
       this.#_handleError(e);
     }
@@ -681,6 +763,106 @@ function normalizeStdioOption(
   }
 }
 
+// Valid stdio string values
+const validStdioStrings = ["ignore", "pipe", "inherit", "overlapped"];
+
+// Result type for getValidStdio
+export interface StdioResult {
+  stdio: Array<{ type: string; fd?: number } | null>;
+  ipc: number | undefined;
+  ipcFd: number | undefined;
+}
+
+/**
+ * Validates and processes stdio configuration.
+ * This is an internal function used by Node.js's child_process module.
+ */
+export function getValidStdio(
+  // deno-lint-ignore no-explicit-any
+  stdio: any,
+  sync?: boolean,
+): StdioResult {
+  let ipc: number | undefined;
+  let ipcFd: number | undefined;
+
+  // If stdio is a string, validate it
+  if (typeof stdio === "string") {
+    if (!validStdioStrings.includes(stdio)) {
+      throw new ERR_INVALID_ARG_VALUE("stdio", stdio);
+    }
+    // Convert string to array
+    stdio = [stdio, stdio, stdio];
+  } else if (!ArrayIsArray(stdio)) {
+    throw new ERR_INVALID_ARG_VALUE("stdio", stdio);
+  }
+
+  // Expand stdio array to at least 3 elements (mutates the input array)
+  while (stdio.length < 3) {
+    ArrayPrototypePush(stdio, undefined);
+  }
+
+  // Process each stdio element
+  const result: Array<{ type: string; fd?: number } | null> = [];
+
+  for (let i = 0; i < stdio.length; i++) {
+    const value = stdio[i];
+
+    if (value === "ipc") {
+      if (sync) {
+        throw new ERR_IPC_SYNC_FORK();
+      }
+      ipc = i;
+      ipcFd = i;
+      result.push({ type: "ipc" });
+    } else if (value === "ignore" || value === null) {
+      result.push({ type: "ignore" });
+    } else if (value === "pipe" || value === undefined) {
+      result.push({ type: "pipe" });
+    } else if (value === "inherit") {
+      result.push({ type: "inherit" });
+    } else if (value === "overlapped") {
+      result.push({ type: "overlapped" });
+    } else if (typeof value === "number") {
+      result.push({ type: "fd", fd: value });
+    } else if (typeof value === "string") {
+      // Invalid string value
+      throw new ERR_INVALID_SYNC_FORK_INPUT(value);
+    } else if (typeof value === "object" && value !== null) {
+      // Check if it's a Stream with fd property (like process.stdin/stdout/stderr)
+      if (
+        value.fd !== undefined && typeof value.fd === "number"
+      ) {
+        result.push({ type: "fd", fd: value.fd });
+      } else if (value instanceof Stream) {
+        // Valid Stream object but without fd
+        result.push({ type: "pipe" });
+      } else {
+        // Invalid object
+        throw new ERR_INVALID_ARG_VALUE("stdio", value);
+      }
+    } else {
+      throw new ERR_INVALID_ARG_VALUE("stdio", value);
+    }
+  }
+
+  return {
+    stdio: result,
+    ipc,
+    ipcFd,
+  };
+}
+
+// Check for null bytes in a string and throw ERR_INVALID_ARG_VALUE if found
+export function validateNullByteNotInArg(value: string, name: string): void {
+  if (StringPrototypeIncludes(value, "\0")) {
+    throw new ERR_INVALID_ARG_VALUE(
+      name,
+      value,
+      "must be a string without null bytes",
+    );
+  }
+}
+
 export function normalizeSpawnArguments(
   file: string,
   args: string[],
@@ -692,6 +874,9 @@ export function normalizeSpawnArguments(
     throw new ERR_INVALID_ARG_VALUE("file", file, "cannot be empty");
   }
 
+  // Check for null bytes in file
+  validateNullByteNotInArg(file, "file");
+
   if (ArrayIsArray(args)) {
     args = ArrayPrototypeSlice(args);
   } else if (args == null) {
@@ -703,6 +888,14 @@ export function normalizeSpawnArguments(
     args = [];
   }
 
+  // Check for null bytes in args
+  for (let i = 0; i < args.length; i++) {
+    const arg = args[i];
+    if (typeof arg === "string") {
+      validateNullByteNotInArg(arg, `args[${i}]`);
+    }
+  }
+
   if (options === undefined) {
     options = kEmptyObject;
   } else {
@@ -714,6 +907,7 @@ export function normalizeSpawnArguments(
   // Validate the cwd, if present.
   if (cwd != null) {
     cwd = getValidatedPath(cwd, "options.cwd") as string;
+    validateNullByteNotInArg(cwd, "options.cwd");
   }
 
   // Validate detached, if present.
@@ -743,10 +937,14 @@ export function normalizeSpawnArguments(
       options.shell,
     );
   }
+  if (typeof options.shell === "string") {
+    validateNullByteNotInArg(options.shell, "options.shell");
+  }
 
   // Validate argv0, if present.
   if (options.argv0 != null) {
     validateString(options.argv0, "options.argv0");
+    validateNullByteNotInArg(options.argv0, "options.argv0");
   }
 
   // Validate windowsHide, if present.
@@ -763,8 +961,16 @@ export function normalizeSpawnArguments(
     );
   }
 
+  validateOneOf(options.serialization, "options.serialization", [
+    undefined,
+    "json",
+    "advanced",
+  ]);
+
   if (options.shell) {
-    const command = ArrayPrototypeJoin([file, ...args], " ");
+    let command = ArrayPrototypeJoin([file, ...args], " ");
+    // Transform Node.js flags to Deno equivalents in shell commands that invoke Deno
+    command = transformDenoShellCommand(command, options.env);
     // Set the shell, switches, and commands.
     if (process.platform === "win32") {
       if (typeof options.shell === "string") {
@@ -834,6 +1040,9 @@ export function normalizeSpawnArguments(
   for (const key of envKeys) {
     const value = env[key];
     if (value !== undefined) {
+      // Check for null bytes in env keys and values
+      validateNullByteNotInArg(key, `options.env['${key}']`);
+      validateNullByteNotInArg(String(value), `options.env['${key}']`);
       ArrayPrototypePush(envPairs, `${key}=${value}`);
     }
   }
@@ -849,6 +1058,7 @@ export function normalizeSpawnArguments(
     file,
     windowsHide: !!options.windowsHide,
     windowsVerbatimArguments: !!windowsVerbatimArguments,
+    serialization: options.serialization || "json",
   };
 }
 
@@ -876,6 +1086,147 @@ function waitForStreamToClose(stream: Stream) {
   return deferred.promise;
 }
 
+/**
+ * Simple shell argument splitter that handles double and single quotes.
+ * Used to parse the arguments portion of a shell command string.
+ */
+function splitShellArgs(str: string): string[] {
+  const args: string[] = [];
+  let current = "";
+  let inDouble = false;
+  let inSingle = false;
+
+  for (let i = 0; i < str.length; i++) {
+    const ch = str[i];
+    if (ch === '"' && !inSingle) {
+      inDouble = !inDouble;
+    } else if (ch === "'" && !inDouble) {
+      inSingle = !inSingle;
+    } else if (ch === " " && !inDouble && !inSingle) {
+      if (current.length > 0) {
+        args.push(current);
+        current = "";
+      }
+    } else {
+      current += ch;
+    }
+  }
+  if (current.length > 0) {
+    args.push(current);
+  }
+  return args;
+}
+
+/**
+ * Transforms a shell command that invokes Deno with Node.js flags into Deno-compatible flags.
+ * Uses the Rust CLI parser (op_node_translate_cli_args) to handle argument translation,
+ * including subcommand detection, -c/--check flag handling, and adding "run -A".
+ */
+function transformDenoShellCommand(
+  command: string,
+  env?: Record<string, string | number | boolean>,
+): string {
+  const denoPath = Deno.execPath();
+
+  // Check if the command starts with the Deno executable (possibly quoted)
+  const quotedDenoPath = `"${denoPath}"`;
+  const singleQuotedDenoPath = `'${denoPath}'`;
+
+  let startsWithDeno = false;
+  let denoPathLength = 0;
+  let shellVarPrefix = "";
+
+  if (command.startsWith(quotedDenoPath)) {
+    startsWithDeno = true;
+    denoPathLength = quotedDenoPath.length;
+  } else if (command.startsWith(singleQuotedDenoPath)) {
+    startsWithDeno = true;
+    denoPathLength = singleQuotedDenoPath.length;
+  } else if (command.startsWith(denoPath)) {
+    startsWithDeno = true;
+    denoPathLength = denoPath.length;
+  } else if (env) {
+    // Check for shell variable that references the Deno path
+    // Pattern: "${VARNAME}" or $VARNAME at start of command
+    const shellVarMatch = command.match(
+      /^(?:"\$\{([^}]+)\}"|\$\{([^}]+)\}|\$([A-Za-z_][A-Za-z0-9_]*))/,
+    );
+    if (shellVarMatch) {
+      const varName = shellVarMatch[1] || shellVarMatch[2] ||
+        shellVarMatch[3];
+      const varValue = env[varName];
+      if (varValue !== undefined && String(varValue) === denoPath) {
+        startsWithDeno = true;
+        shellVarPrefix = shellVarMatch[0];
+        denoPathLength = shellVarMatch[0].length;
+      }
+    }
+  }
+
+  if (!startsWithDeno) {
+    return command;
+  }
+
+  // Extract the rest of the command after the Deno path
+  const rest = command.slice(denoPathLength).trimStart();
+
+  if (rest.length === 0) {
+    return command;
+  }
+
+  // Split the remaining args and use the Rust parser to translate them
+  const args = splitShellArgs(rest);
+
+  try {
+    const result = op_node_translate_cli_args(args, false);
+    // Check if any translated arg contains shell metacharacters (e.g. from eval
+    // wrapping). If so, the result can't be safely used in a shell command.
+    for (let i = 0; i < result.deno_args.length; i++) {
+      if (/[();&|<>`!]/.test(result.deno_args[i])) {
+        return command;
+      }
+    }
+    const prefix = shellVarPrefix || command.slice(0, denoPathLength);
+    return prefix + " " + result.deno_args.join(" ");
+  } catch {
+    // If the Rust parser fails (unknown flags), return the original command
+    return command;
+  }
+}
+
+/**
+ * Find the first non-flag argument in a list of command line arguments.
+ * This is used to determine if the user is spawning a Deno subcommand
+ * or a script, and to check if the script is in an npm package.
+ */
+function findFirstNonFlagArg(args: string[]): string | null {
+  for (let i = 0; i < args.length; i++) {
+    const arg = args[i];
+    // Stop at '--' - everything after is positional
+    if (arg === "--") {
+      return i + 1 < args.length ? args[i + 1] : null;
+    }
+    // If it doesn't start with '-', it's a positional argument
+    if (!arg.startsWith("-")) {
+      return arg;
+    }
+    // Skip known flags that take a value
+    if (
+      arg === "-e" || arg === "--eval" ||
+      arg === "-p" || arg === "--print" ||
+      arg === "-r" || arg === "--require" ||
+      arg === "-C" || arg === "--conditions" ||
+      arg === "-c" || arg === "--check" ||
+      arg === "--import" ||
+      arg === "--loader" ||
+      arg === "--experimental-loader"
+    ) {
+      i++; // Skip the next arg (the value)
+    }
+  }
+  return null;
+}
+
 /**
  * This function is based on https://github.com/nodejs/node/blob/fc6426ccc4b4cb73076356fb6dbf46a28953af01/lib/child_process.js#L504-L528.
  * Copyright Joyent, Inc. and other Node contributors. All rights reserved. MIT license.
@@ -883,18 +1234,28 @@ function waitForStreamToClose(stream: Stream) {
 function buildCommand(
   file: string,
   args: string[],
-  shell: string | boolean,
   env: Record<string, string | number | boolean>,
 ): [string, string[], boolean] {
   let includeNpmProcessState = false;
   if (file === Deno.execPath()) {
-    let nodeOptions: string[];
-    // The user is trying to spawn another Deno process as Node.js.
-    [args, nodeOptions, includeNpmProcessState] = toDenoArgs(args);
+    // Ensure all args are strings (Node allows numbers in args array)
+    args = args.map((arg) => String(arg));
 
-    // Update NODE_OPTIONS if it exists
-    if (nodeOptions.length > 0) {
-      const options = nodeOptions.join(" ");
+    // Find script path to check if it's in an npm package
+    const firstNonFlagArg = findFirstNonFlagArg(args);
+    const scriptInNpmPackage = firstNonFlagArg !== null
+      ? op_node_in_npm_package(firstNonFlagArg)
+      : false;
+
+    // Use the Rust parser to translate Node.js args to Deno args
+    // The parser handles Deno-style args (e.g., "run -A script.js") by passing them through unchanged
+    const result = op_node_translate_cli_args(args, scriptInNpmPackage);
+    args = result.deno_args;
+    includeNpmProcessState = result.needs_npm_process_state;
+
+    // Update NODE_OPTIONS if needed
+    if (result.node_options.length > 0) {
+      const options = result.node_options.join(" ");
       if (env.NODE_OPTIONS) {
         env.NODE_OPTIONS += " " + options;
       } else {
@@ -903,43 +1264,19 @@ function buildCommand(
     }
   }
 
-  if (shell) {
-    const command = [file, ...args].join(" ");
-
-    // Set the shell, switches, and commands.
-    if (isWindows) {
-      if (typeof shell === "string") {
-        file = shell;
-      } else {
-        file = Deno.env.get("comspec") || "cmd.exe";
-      }
-      // '/d /s /c' is used only for cmd.exe.
-      if (/^(?:.*\\)?cmd(?:\.exe)?$/i.test(file)) {
-        args = ["/d", "/s", "/c", `"${command}"`];
-      } else {
-        args = ["-c", command];
-      }
-    } else {
-      if (typeof shell === "string") {
-        file = shell;
-      } else {
-        file = "/bin/sh";
-      }
-      args = ["-c", command];
-    }
-  }
-
   return [file, args, includeNpmProcessState];
 }
 
-function _createSpawnSyncError(
+function _createSpawnError(
   status: string,
   command: string,
   args: string[] = [],
+  sync: boolean = false,
 ): ErrnoException {
+  const syscall = sync ? "spawnSync " : "spawn ";
   const error = errnoException(
     codeMap.get(status),
-    "spawnSync " + command,
+    syscall + command,
   );
   error.path = command;
   error.spawnargs = args;
@@ -1033,7 +1370,6 @@ export function spawnSync(
     env = Deno.env.toObject(),
     input,
     stdio = ["pipe", "pipe", "pipe"],
-    shell = false,
     cwd,
     encoding,
     uid,
@@ -1048,10 +1384,11 @@ export function spawnSync(
     _channel, // TODO(kt3k): handle this correctly
   ] = normalizeStdioOption(stdio);
   let includeNpmProcessState = false;
+  // Skip argv0 when calling buildCommand (same as #spawnInternal)
+  const argsToProcess = args && args.length > 0 ? args.slice(1) : [];
   [command, args, includeNpmProcessState] = buildCommand(
     command,
-    args ?? [],
-    shell,
+    argsToProcess,
     env,
   );
   const input_ = normalizeInput(input);
@@ -1082,7 +1419,7 @@ export function spawnSync(
       (stdout && stdout.length > maxBuffer!) ||
       (stderr && stderr.length > maxBuffer!)
     ) {
-      result.error = _createSpawnSyncError("ENOBUFS", command, args);
+      result.error = _createSpawnError("ENOBUFS", command, args, true);
     }
 
     if (encoding && encoding !== "buffer") {
@@ -1097,231 +1434,12 @@ export function spawnSync(
     result.output = [output.signal, stdout, stderr];
   } catch (err) {
     if (err instanceof Deno.errors.NotFound) {
-      result.error = _createSpawnSyncError("ENOENT", command, args);
+      result.error = _createSpawnError("ENOENT", command, args, true);
     }
   }
   return result;
 }
 
-// These are Node.js CLI flags that expect a value. It's necessary to
-// understand these flags in order to properly replace flags passed to the
-// child process. For example, -e is a Node flag for eval mode if it is part
-// of process.execArgv. However, -e could also be an application flag if it is
-// part of process.execv instead. We only want to process execArgv flags.
-const kLongArgType = 1;
-const kShortArgType = 2;
-const kLongArg = { type: kLongArgType };
-const kShortArg = { type: kShortArgType };
-const kNodeFlagsMap = new Map([
-  ["--build-snapshot", kLongArg],
-  ["-c", kShortArg],
-  ["--check", kLongArg],
-  ["-C", kShortArg],
-  ["--conditions", kLongArg],
-  ["--cpu-prof-dir", kLongArg],
-  ["--cpu-prof-interval", kLongArg],
-  ["--cpu-prof-name", kLongArg],
-  ["--diagnostic-dir", kLongArg],
-  ["--disable-proto", kLongArg],
-  ["--dns-result-order", kLongArg],
-  ["-e", kShortArg],
-  ["--eval", kLongArg],
-  ["--experimental-loader", kLongArg],
-  ["--experimental-policy", kLongArg],
-  ["--experimental-specifier-resolution", kLongArg],
-  ["--heapsnapshot-near-heap-limit", kLongArg],
-  ["--heapsnapshot-signal", kLongArg],
-  ["--heap-prof-dir", kLongArg],
-  ["--heap-prof-interval", kLongArg],
-  ["--heap-prof-name", kLongArg],
-  ["--icu-data-dir", kLongArg],
-  ["--input-type", kLongArg],
-  ["--inspect-publish-uid", kLongArg],
-  ["--max-http-header-size", kLongArg],
-  ["--openssl-config", kLongArg],
-  ["-p", kShortArg],
-  ["--print", kLongArg],
-  ["--policy-integrity", kLongArg],
-  ["--prof-process", kLongArg],
-  ["-r", kShortArg],
-  ["--require", kLongArg],
-  ["--redirect-warnings", kLongArg],
-  ["--report-dir", kLongArg],
-  ["--report-directory", kLongArg],
-  ["--report-filename", kLongArg],
-  ["--report-signal", kLongArg],
-  ["--secure-heap", kLongArg],
-  ["--secure-heap-min", kLongArg],
-  ["--snapshot-blob", kLongArg],
-  ["--title", kLongArg],
-  ["--tls-cipher-list", kLongArg],
-  ["--tls-keylog", kLongArg],
-  ["--unhandled-rejections", kLongArg],
-  ["--use-largepages", kLongArg],
-  ["--v8-pool-size", kLongArg],
-]);
-const kDenoSubcommands = new Set([
-  "add",
-  "bench",
-  "cache",
-  "check",
-  "compile",
-  "completions",
-  "coverage",
-  "doc",
-  "eval",
-  "fmt",
-  "help",
-  "info",
-  "init",
-  "install",
-  "lint",
-  "lsp",
-  "publish",
-  "repl",
-  "run",
-  "tasks",
-  "test",
-  "types",
-  "uninstall",
-  "upgrade",
-  "vendor",
-]);
-
-/** Wraps the script for (Node.js) --eval / --print argument
- * Note: Builtin modules are available as global variables */
-function wrapScriptForEval(sourceCode: string): string {
-  // Note: We need vm.runInThisContext call here to get the last evaluated
-  // value of the source with multiple statements. `deno eval -p` surrounds
-  // the source code like `console.log(${source})`, and it only allows a
-  // single expression.
-  return `
-    process.getBuiltinModule("module").builtinModules
-      .filter((m) => !/\\/|crypto|process/.test(m))
-      .forEach((m) => { globalThis[m] = process.getBuiltinModule(m); }),
-    vm.runInThisContext(${JSON.stringify(sourceCode)})
-  `;
-}
-
-/** Returns deno args and NODE_OPTIONS for simulating Node.js cli */
-function toDenoArgs(args: string[]): [string[], string[], boolean] {
-  if (args.length === 0) {
-    return [args, args, false];
-  }
-
-  // Update this logic as more CLI arguments are mapped from Node to Deno.
-  const denoArgs: string[] = [];
-  const nodeOptions: string[] = [];
-  let useRunArgs = true;
-  let needsNpmProcessState = false;
-
-  for (let i = 0; i < args.length; i++) {
-    const arg = args[i];
-
-    if (arg.charAt(0) !== "-" || arg === "--") {
-      // Not a flag or no more arguments.
-
-      // If the arg is a Deno subcommand, then the child process is being
-      // spawned as Deno, not Deno in Node compat mode. In this case, bail out
-      // and return the original args.
-      if (kDenoSubcommands.has(arg)) {
-        return [args, [], false];
-      }
-
-      // if the user is launching a script in the node_modules or
-      // global cache, include the process state
-      needsNpmProcessState = op_node_in_npm_package(arg);
-
-      // Copy of the rest of the arguments to the output.
-      for (let j = i; j < args.length; j++) {
-        denoArgs.push(args[j]);
-      }
-
-      break;
-    }
-
-    // Something that looks like a flag was passed.
-    let flag = arg;
-    let flagInfo = kNodeFlagsMap.get(arg);
-    let isLongWithValue = false;
-    let flagValue;
-
-    if (flag === "--v8-options") {
-      // If --v8-options is passed, it should be replaced with --v8-flags="--help".
-      denoArgs.push("--v8-flags=--help");
-      continue;
-    }
-
-    if (flagInfo === undefined) {
-      // If the flag was not found, it's either not a known flag or it's a long
-      // flag containing an '='.
-      const splitAt = arg.indexOf("=");
-
-      if (splitAt !== -1) {
-        flag = arg.slice(0, splitAt);
-        flagInfo = kNodeFlagsMap.get(flag);
-        flagValue = arg.slice(splitAt + 1);
-        isLongWithValue = true;
-      }
-    }
-
-    if (flagInfo === undefined) {
-      if (arg === "--no-warnings") {
-        denoArgs.push("--quiet");
-        nodeOptions.push(arg);
-      } else if (arg === "--expose-internals") {
-        // internals are always exposed in Deno.
-      } else if (arg === "--permission") {
-        // ignore --permission flag
-      } else if (arg === "--pending-deprecation") {
-        nodeOptions.push(arg);
-      } else if (StringPrototypeStartsWith(arg, "--experimental-")) {
-        // `--experimental-*` args are ignored, because most experimental Node features
-        // are implemented in Deno, but it doens't exactly match Deno's `--unstable-*` flags.
-      } else {
-        // Not a known flag that expects a value. Just copy it to the output.
-        denoArgs.push(arg);
-      }
-      continue;
-    }
-
-    // This is a flag with a value. Get the value if we don't already have it.
-    if (flagValue === undefined) {
-      i++;
-
-      if (i >= args.length) {
-        // There was user error. There should be another arg for the value, but
-        // there isn't one. Just copy the arg to the output. It's not going
-        // to work anyway.
-        denoArgs.push(arg);
-        continue;
-      }
-
-      flagValue = args[i];
-    }
-
-    // Remap Node's eval flags to Deno.
-    if (flag === "-e" || flag === "--eval") {
-      denoArgs.push("eval", wrapScriptForEval(flagValue));
-      useRunArgs = false;
-    } else if (flag === "-p" || flag === "--print") {
-      denoArgs.push("eval", "-p", wrapScriptForEval(flagValue));
-      useRunArgs = false;
-    } else if (isLongWithValue) {
-      denoArgs.push(arg);
-    } else {
-      denoArgs.push(flag, flagValue);
-    }
-  }
-
-  if (useRunArgs) {
-    // -A is not ideal, but needed to propagate permissions.
-    denoArgs.unshift("run", "-A");
-  }
-
-  return [denoArgs, nodeOptions, needsNpmProcessState];
-}
-
 const kControlDisconnect = Symbol("kControlDisconnect");
 const kPendingMessages = Symbol("kPendingMessages");
 
@@ -1332,20 +1450,22 @@ class Control extends EventEmitter {
   #refExplicitlySet = false;
   #connected = true;
   [kPendingMessages] = [];
-  constructor(channel: number) {
+  #serialization: "json" | "advanced";
+  constructor(channel: number, serialization: "json" | "advanced") {
     super();
     this.#channel = channel;
+    this.#serialization = serialization;
   }
 
   #ref() {
     if (this.#connected) {
-      op_node_ipc_ref(this.#channel);
+      op_node_ipc_ref(this.#channel, this.#serialization === "json");
     }
   }
 
   #unref() {
     if (this.#connected) {
-      op_node_ipc_unref(this.#channel);
+      op_node_ipc_unref(this.#channel, this.#serialization === "json");
     }
   }
 
@@ -1397,18 +1517,37 @@ function internalCmdName(msg: InternalMessage): string {
   return StringPrototypeSlice(msg.cmd, 5);
 }
 
-// deno-lint-ignore no-explicit-any
-export function setupChannel(target: any, ipc: number) {
-  const control = new Control(ipc);
+let hasSetBufferConstructor = false;
+
+export function setupChannel(
+  // deno-lint-ignore no-explicit-any
+  target: any,
+  ipc: number,
+  serialization: "json" | "advanced",
+) {
+  const control = new Control(ipc, serialization);
   target.channel = control;
 
+  if (!hasSetBufferConstructor) {
+    op_node_ipc_buffer_constructor(Buffer, FastBuffer.prototype);
+    hasSetBufferConstructor = true;
+  }
+
+  const writeFn = serialization === "json"
+    ? op_node_ipc_write_json
+    : op_node_ipc_write_advanced;
+  const readFn = serialization === "json"
+    ? op_node_ipc_read_json
+    : op_node_ipc_read_advanced;
+
   async function readLoop() {
     try {
       while (true) {
         if (!target.connected || target.killed) {
           return;
         }
-        const prom = op_node_ipc_read(ipc);
+        // TODO(nathanwhit): maybe allow returning multiple messages in a single read? needs benchmarking.
+        const prom = readFn(ipc);
         // there will always be a pending read promise,
         // but it shouldn't keep the event loop from exiting
         core.unrefOpPromise(prom);
@@ -1500,7 +1639,7 @@ export function setupChannel(target: any, ipc: number) {
     // this acts as a backpressure mechanism.
     const queueOk = [true];
     control.refCounted();
-    op_node_ipc_write(ipc, message, queueOk)
+    writeFn(ipc, message, queueOk)
       .then(() => {
         control.unrefCounted();
         if (callback) {
@@ -1548,6 +1687,7 @@ export function setupChannel(target: any, ipc: number) {
 
 export default {
   ChildProcess,
+  getValidStdio,
   normalizeSpawnArguments,
   stdioStringToArray,
   spawnSync,
