@@ -1,4 +1,4 @@
-// Copyright 2018-2025 the Deno authors. MIT license.
+// Copyright 2018-2026 the Deno authors. MIT license.
 // Copyright Joyent, Inc. and other Node contributors.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
@@ -40,6 +40,7 @@ import {
   op_dns_resolve,
   op_net_get_ips_from_perm_token,
   op_node_getaddrinfo,
+  op_node_getnameinfo,
 } from "ext:core/ops";
 
 interface LookupAddress {
@@ -137,6 +138,44 @@ export function getaddrinfo(
   return 0;
 }
 
+export class GetNameInfoReqWrap extends AsyncWrap {
+  address!: string;
+  port!: number;
+
+  callback?: (
+    err: ErrnoException | null,
+    hostname?: string,
+    service?: string,
+  ) => void;
+  resolve!: (result: { hostname: string; service: string }) => void;
+  reject!: (err: ErrnoException | null) => void;
+  oncomplete!: (
+    err: Error | null,
+    hostname?: string,
+    service?: string,
+  ) => void;
+
+  constructor() {
+    super(providerType.GETNAMEINFOREQWRAP);
+  }
+}
+
+export function getnameinfo(
+  req: GetNameInfoReqWrap,
+  address: string,
+  port: number,
+): number {
+  (async () => {
+    try {
+      const [hostname, service] = await op_node_getnameinfo(address, port);
+      req.oncomplete(null, hostname, service);
+    } catch (err) {
+      req.oncomplete(err as Error);
+    }
+  })();
+  return 0;
+}
+
 export class QueryReqWrap extends AsyncWrap {
   bindingName!: string;
   hostname!: string;
@@ -243,7 +282,7 @@ export class ChannelWrap extends AsyncWrap implements ChannelWrapQuery {
         query,
         recordType,
         options: resolveOptions,
-      });
+      }, /* useEdns0 */ false);
       if (ttl) {
         ret = res;
       } else {
@@ -576,6 +615,7 @@ export default {
   DNS_ORDER_IPV6_FIRST,
   GetAddrInfoReqWrap,
   getaddrinfo,
+  getnameinfo,
   QueryReqWrap,
   ChannelWrap,
   strerror,
