@@ -1,4 +1,4 @@
-// Copyright 2018-2025 the Deno authors. MIT license.
+// Copyright 2018-2026 the Deno authors. MIT license.
 // Copyright Joyent, Inc. and Node.js contributors. All rights reserved. MIT license.
 
 // TODO(petamoriken): enable prefer-primordials for node polyfills
@@ -12,6 +12,10 @@ import {
   op_geteuid,
   op_node_load_env_file,
   op_node_process_kill,
+  op_node_process_setegid,
+  op_node_process_seteuid,
+  op_node_process_setgid,
+  op_node_process_setuid,
   op_process_abort,
 } from "ext:core/ops";
 
@@ -24,8 +28,10 @@ import {
   validateNumber,
   validateObject,
   validateString,
+  validateUint32,
 } from "ext:deno_node/internal/validators.mjs";
 import {
+  denoErrorToNodeError,
   ERR_INVALID_ARG_TYPE,
   ERR_INVALID_ARG_VALUE_RANGE,
   ERR_OUT_OF_RANGE,
@@ -33,7 +39,7 @@ import {
   errnoException,
 } from "ext:deno_node/internal/errors.ts";
 import { getOptionValue } from "ext:deno_node/internal/options.ts";
-import { assert } from "ext:deno_node/_util/asserts.ts";
+import assert from "node:assert";
 import { join } from "node:path";
 import { pathFromURL } from "ext:deno_web/00_infra.js";
 import {
@@ -64,7 +70,8 @@ import {
   processTicksAndRejections,
   runNextTicks,
 } from "ext:deno_node/_next_tick.ts";
-import { isWindows } from "ext:deno_node/_util/os.ts";
+import { runImmediates } from "ext:deno_node/internal/timers.mjs";
+import { isAndroid, isWindows } from "ext:deno_node/_util/os.ts";
 import * as io from "ext:deno_io/12_io.js";
 import * as denoOs from "ext:deno_os/30_os.js";
 
@@ -75,6 +82,7 @@ export let arch = "";
 export let platform = isWindows ? "win32" : ""; // initialized during bootstrap
 
 export let pid = 0;
+export let ppid = 0;
 
 let stdin, stdout, stderr;
 
@@ -86,7 +94,7 @@ import * as uv from "ext:deno_node/internal_binding/uv.ts";
 import type { BindingName } from "ext:deno_node/internal_binding/mod.ts";
 import { buildAllowedFlags } from "ext:deno_node/internal/process/per_thread.mjs";
 
-const { NumberMAX_SAFE_INTEGER } = primordials;
+const { NumberMAX_SAFE_INTEGER, ObjectDefineProperty } = primordials;
 
 const notImplementedEvents = [
   "multipleResolves",
@@ -374,16 +382,43 @@ export function kill(pid: number, sig: string | number = "SIGTERM") {
   return true;
 }
 
-let getgid, getuid, getegid, geteuid;
+let getgid, getuid, getegid, geteuid, setegid, seteuid, setgid, setuid;
+
+function wrapIdSetter(
+  syscall: string,
+  fn: (id: number | string) => void,
+): (id: number | string) => void {
+  return function (id: number | string) {
+    if (typeof id === "number") {
+      validateUint32(id, "id");
+      id >>>= 0;
+    } else if (typeof id !== "string") {
+      throw new ERR_INVALID_ARG_TYPE("id", ["number", "string"], id);
+    }
+
+    try {
+      fn(id);
+    } catch (err) {
+      throw denoErrorToNodeError(err as Error, { syscall });
+    }
+  };
+}
 
 if (!isWindows) {
   getgid = () => Deno.gid();
   getuid = () => Deno.uid();
   getegid = () => op_getegid();
   geteuid = () => op_geteuid();
+
+  if (!isAndroid) {
+    setegid = wrapIdSetter("setegid", op_node_process_setegid);
+    seteuid = wrapIdSetter("seteuid", op_node_process_seteuid);
+    setgid = wrapIdSetter("setgid", op_node_process_setgid);
+    setuid = wrapIdSetter("setuid", op_node_process_setuid);
+  }
 }
 
-export { getegid, geteuid, getgid, getuid };
+export { getegid, geteuid, getgid, getuid, setegid, seteuid, setgid, setuid };
 
 const ALLOWED_FLAGS = buildAllowedFlags();
 
@@ -722,6 +757,15 @@ process.setSourceMapsEnabled = (_val: boolean) => {
   // TODO(@satyarohith): support disabling source maps if needed.
 };
 
+// Source maps are always enabled in Deno.
+Object.defineProperty(process, "sourceMapsEnabled", {
+  get() {
+    return true; // Source maps are always enabled in Deno.
+  },
+  enumerable: true,
+  configurable: true,
+});
+
 /**
  * Returns the current high-resolution real time in a [seconds, nanoseconds]
  * tuple.
@@ -792,6 +836,18 @@ process.getegid = getegid;
 /** This method is removed on Windows */
 process.geteuid = geteuid;
 
+/** This method is removed on Windows */
+process.setegid = setegid;
+
+/** This method is removed on Windows */
+process.seteuid = seteuid;
+
+/** This method is removed on Windows */
+process.setgid = setgid;
+
+/** This method is removed on Windows */
+process.setuid = setuid;
+
 process.getBuiltinModule = getBuiltinModule;
 
 // TODO(kt3k): Implement this when we added -e option to node compat mode
@@ -828,7 +884,40 @@ Object.defineProperty(process, "allowedNodeEnvironmentFlags", {
 
 export const allowedNodeEnvironmentFlags = ALLOWED_FLAGS;
 
-process.features = { inspector: false };
+const features = {
+  inspector: true,
+  // TODO(bartlomieju): not sure if it's worth getting actual value during build process
+  debug: false,
+  uv: true,
+  ipv6: true,
+  // deno-lint-ignore camelcase
+  tls_alpn: true,
+  // deno-lint-ignore camelcase
+  tls_sni: true,
+  // deno-lint-ignore camelcase
+  tls_ocsp: true,
+  tls: true,
+  // deno-lint-ignore camelcase
+  openssl_is_boringssl: false,
+  // deno-lint-ignore camelcase
+  cached_builtins: true,
+  // deno-lint-ignore camelcase
+  require_module: true,
+  get typescript() {
+    if (Deno.build.standalone) {
+      return false;
+    }
+    return "transform";
+  },
+};
+
+ObjectDefineProperty(process, "features", {
+  __proto__: null,
+  enumerable: true,
+  writable: false,
+  configurable: false,
+  value: features,
+});
 
 // TODO(kt3k): Get the value from --no-deprecation flag.
 process.noDeprecation = false;
@@ -969,18 +1058,6 @@ function synchronizeListeners() {
   }
 }
 
-// Overwrites the 1st and 2nd items with getters.
-Object.defineProperty(argv, "0", { get: () => argv0 });
-Object.defineProperty(argv, "1", {
-  get: () => {
-    if (Deno.mainModule?.startsWith("file:")) {
-      return pathFromURL(new URL(Deno.mainModule));
-    } else {
-      return join(Deno.cwd(), "$deno$node.mjs");
-    }
-  },
-});
-
 internals.dispatchProcessBeforeExitEvent = dispatchProcessBeforeExitEvent;
 internals.dispatchProcessExitEvent = dispatchProcessExitEvent;
 // Should be called only once, in `runtime/js/99_main.js` when the runtime is
@@ -994,6 +1071,10 @@ internals.__bootstrapNodeProcess = function (
 ) {
   if (!warmup) {
     argv0 = argv0Val || "";
+    argv[0] = argv0;
+    argv[1] = Deno.mainModule?.startsWith("file:")
+      ? pathFromURL(new URL(Deno.mainModule))
+      : join(Deno.cwd(), "$deno$node.mjs");
     // Manually concatenate these arrays to avoid triggering the getter
     for (let i = 0; i < args.length; i++) {
       argv[i + 2] = args[i];
@@ -1004,6 +1085,7 @@ internals.__bootstrapNodeProcess = function (
     }
 
     core.setNextTickCallback(processTicksAndRejections);
+    core.setImmediateCallback(runImmediates);
     core.setMacrotaskCallback(runNextTicks);
     enableNextTick();
 
@@ -1027,7 +1109,7 @@ internals.__bootstrapNodeProcess = function (
     arch = arch_();
     platform = isWindows ? "win32" : Deno.build.os;
     pid = Deno.pid;
-
+    ppid = Deno.ppid;
     initializeDebugEnv(nodeDebug);
 
     if (getOptionValue("--warnings")) {
