@@ -1,4 +1,4 @@
-// Copyright 2018-2025 the Deno authors. MIT license.
+// Copyright 2018-2026 the Deno authors. MIT license.
 
 import { primordials } from "ext:core/mod.js";
 import type { Span } from "ext:deno_telemetry/telemetry.ts";
@@ -32,6 +32,14 @@ export function updateSpanFromResponse(span: Span, response: Response) {
 
 // deno-lint-ignore no-explicit-any
 export function updateSpanFromError(span: Span, error: any) {
-  span.setAttribute("error.type", error.name ?? "Error");
+  const errorType = error.name ?? "Error";
+  span.setAttribute("error.type", errorType);
+  span.setAttribute("exception.type", errorType);
+  if (error.message != null) {
+    span.setAttribute("exception.message", error.message);
+  }
+  if (error.stack != null) {
+    span.setAttribute("exception.stacktrace", error.stack);
+  }
   span.setStatus({ code: 2, message: error.message ?? String(error) });
 }
