@@ -1,4 +1,4 @@
-// Copyright 2018-2025 the Deno authors. MIT license.
+// Copyright 2018-2026 the Deno authors. MIT license.
 
 // NOTE(bartlomieju): some fields are marked as never read, even though they are
 // actually used in the CLI.
@@ -6,12 +6,6 @@
 
 use crate::structs::UnstableFeatureKind;
 
-#[derive(Clone, Debug)]
-pub enum ConfigFileOption {
-  SameAsFlagName,
-  Renamed(&'static str),
-}
-
 #[derive(Clone, Debug)]
 pub struct UnstableFeatureDescription {
   pub name: &'static str,
@@ -20,7 +14,6 @@ pub struct UnstableFeatureDescription {
   pub show_in_help: bool,
   pub kind: UnstableFeatureKind,
   pub env_var: Option<&'static str>,
-  pub config_option: ConfigFileOption,
 }
 
 pub static FEATURE_DESCRIPTIONS: &[UnstableFeatureDescription] = &[
@@ -29,15 +22,13 @@ pub static FEATURE_DESCRIPTIONS: &[UnstableFeatureDescription] = &[
     help_text: "Enable unstable bare node builtins feature",
     show_in_help: true,
     kind: UnstableFeatureKind::Cli,
-    config_option: ConfigFileOption::SameAsFlagName,
     env_var: Some("DENO_UNSTABLE_BARE_NODE_BUILTINS"),
   },
   UnstableFeatureDescription {
     name: "broadcast-channel",
     help_text: "Enable unstable `BroadcastChannel` API",
-    show_in_help: true,
+    show_in_help: false,
     kind: UnstableFeatureKind::Runtime,
-    config_option: ConfigFileOption::SameAsFlagName,
     env_var: None,
   },
   UnstableFeatureDescription {
@@ -45,7 +36,6 @@ pub static FEATURE_DESCRIPTIONS: &[UnstableFeatureDescription] = &[
     help_text: "",
     show_in_help: false,
     kind: UnstableFeatureKind::Cli,
-    config_option: ConfigFileOption::SameAsFlagName,
     env_var: None,
   },
   UnstableFeatureDescription {
@@ -53,7 +43,6 @@ pub static FEATURE_DESCRIPTIONS: &[UnstableFeatureDescription] = &[
     help_text: "Enable unstable `Deno.cron` API",
     show_in_help: true,
     kind: UnstableFeatureKind::Runtime,
-    config_option: ConfigFileOption::SameAsFlagName,
     env_var: None,
   },
   UnstableFeatureDescription {
@@ -61,7 +50,6 @@ pub static FEATURE_DESCRIPTIONS: &[UnstableFeatureDescription] = &[
     help_text: "Treats ambiguous .js, .jsx, .ts, .tsx files as CommonJS modules in more cases",
     show_in_help: true,
     kind: UnstableFeatureKind::Cli,
-    config_option: ConfigFileOption::SameAsFlagName,
     env_var: None,
   },
   UnstableFeatureDescription {
@@ -69,7 +57,6 @@ pub static FEATURE_DESCRIPTIONS: &[UnstableFeatureDescription] = &[
     help_text: "Enable unstable FFI APIs",
     show_in_help: false,
     kind: UnstableFeatureKind::Runtime,
-    config_option: ConfigFileOption::SameAsFlagName,
     env_var: None,
   },
   UnstableFeatureDescription {
@@ -77,7 +64,6 @@ pub static FEATURE_DESCRIPTIONS: &[UnstableFeatureDescription] = &[
     help_text: "Enable unstable file system APIs",
     show_in_help: false,
     kind: UnstableFeatureKind::Runtime,
-    config_option: ConfigFileOption::SameAsFlagName,
     env_var: None,
   },
   UnstableFeatureDescription {
@@ -85,7 +71,6 @@ pub static FEATURE_DESCRIPTIONS: &[UnstableFeatureDescription] = &[
     help_text: "Enable unstable HTTP APIs",
     show_in_help: false,
     kind: UnstableFeatureKind::Runtime,
-    config_option: ConfigFileOption::SameAsFlagName,
     env_var: None,
   },
   UnstableFeatureDescription {
@@ -93,7 +78,6 @@ pub static FEATURE_DESCRIPTIONS: &[UnstableFeatureDescription] = &[
     help_text: "Enable unstable KV APIs",
     show_in_help: true,
     kind: UnstableFeatureKind::Runtime,
-    config_option: ConfigFileOption::SameAsFlagName,
     env_var: None,
   },
   UnstableFeatureDescription {
@@ -101,7 +85,6 @@ pub static FEATURE_DESCRIPTIONS: &[UnstableFeatureDescription] = &[
     help_text: "Lazily loads statically analyzable dynamic imports when not running with type checking. Warning: This may change the order of semver specifier resolution.",
     show_in_help: true,
     kind: UnstableFeatureKind::Cli,
-    config_option: ConfigFileOption::SameAsFlagName,
     env_var: Some("DENO_UNSTABLE_LAZY_DYNAMIC_IMPORTS"),
   },
   UnstableFeatureDescription {
@@ -109,7 +92,6 @@ pub static FEATURE_DESCRIPTIONS: &[UnstableFeatureDescription] = &[
     help_text: "Enable unstable lockfile v5",
     show_in_help: true,
     kind: UnstableFeatureKind::Cli,
-    config_option: ConfigFileOption::SameAsFlagName,
     env_var: Some("DENO_UNSTABLE_LOCKFILE_V5"),
   },
   UnstableFeatureDescription {
@@ -117,7 +99,6 @@ pub static FEATURE_DESCRIPTIONS: &[UnstableFeatureDescription] = &[
     help_text: "enable unstable net APIs",
     show_in_help: true,
     kind: UnstableFeatureKind::Runtime,
-    config_option: ConfigFileOption::SameAsFlagName,
     env_var: None,
   },
   UnstableFeatureDescription {
@@ -125,7 +106,6 @@ pub static FEATURE_DESCRIPTIONS: &[UnstableFeatureDescription] = &[
     help_text: "Enable abort signal in Deno.serve without legacy behavior. This will not abort the server when the request is handled successfully.",
     show_in_help: true,
     kind: UnstableFeatureKind::Runtime,
-    config_option: ConfigFileOption::SameAsFlagName,
     env_var: None,
   },
   UnstableFeatureDescription {
@@ -133,7 +113,6 @@ pub static FEATURE_DESCRIPTIONS: &[UnstableFeatureDescription] = &[
     help_text: "Prefer Node.js globals over Deno globals - currently this refers to `setTimeout` and `setInterval` APIs.",
     show_in_help: true,
     kind: UnstableFeatureKind::Runtime,
-    config_option: ConfigFileOption::SameAsFlagName,
     env_var: None,
   },
   UnstableFeatureDescription {
@@ -141,7 +120,6 @@ pub static FEATURE_DESCRIPTIONS: &[UnstableFeatureDescription] = &[
     help_text: "Enable unstable lazy caching of npm dependencies, downloading them only as needed (disabled: all npm packages in package.json are installed on startup; enabled: only npm packages that are actually referenced in an import are installed",
     show_in_help: true,
     kind: UnstableFeatureKind::Cli,
-    config_option: ConfigFileOption::SameAsFlagName,
     env_var: Some("DENO_UNSTABLE_NPM_LAZY_CACHING"),
   },
   UnstableFeatureDescription {
@@ -149,7 +127,6 @@ pub static FEATURE_DESCRIPTIONS: &[UnstableFeatureDescription] = &[
     help_text: "Enable unstable OpenTelemetry features",
     show_in_help: false,
     kind: UnstableFeatureKind::Runtime,
-    config_option: ConfigFileOption::SameAsFlagName,
     env_var: None,
   },
   UnstableFeatureDescription {
@@ -157,7 +134,6 @@ pub static FEATURE_DESCRIPTIONS: &[UnstableFeatureDescription] = &[
     help_text: "Enable unstable process APIs",
     show_in_help: false,
     kind: UnstableFeatureKind::Runtime,
-    config_option: ConfigFileOption::SameAsFlagName,
     env_var: None,
   },
   UnstableFeatureDescription {
@@ -165,7 +141,6 @@ pub static FEATURE_DESCRIPTIONS: &[UnstableFeatureDescription] = &[
     help_text: "Enable unstable 'bytes' and 'text' imports.",
     show_in_help: true,
     kind: UnstableFeatureKind::Runtime,
-    config_option: ConfigFileOption::SameAsFlagName,
     env_var: Some("DENO_UNSTABLE_RAW_IMPORTS"),
   },
   UnstableFeatureDescription {
@@ -173,7 +148,6 @@ pub static FEATURE_DESCRIPTIONS: &[UnstableFeatureDescription] = &[
     help_text: "Enable unstable resolving of specifiers by extension probing, .js to .ts, and directory probing",
     show_in_help: true,
     kind: UnstableFeatureKind::Cli,
-    config_option: ConfigFileOption::SameAsFlagName,
     env_var: Some("DENO_UNSTABLE_SLOPPY_IMPORTS"),
   },
   UnstableFeatureDescription {
@@ -181,7 +155,6 @@ pub static FEATURE_DESCRIPTIONS: &[UnstableFeatureDescription] = &[
     help_text: "Enable subdomain wildcards support for the `--allow-net` flag",
     show_in_help: false,
     kind: UnstableFeatureKind::Cli,
-    config_option: ConfigFileOption::SameAsFlagName,
     env_var: Some("DENO_UNSTABLE_SUBDOMAIN_WILDCARDS"),
   },
   UnstableFeatureDescription {
@@ -189,7 +162,6 @@ pub static FEATURE_DESCRIPTIONS: &[UnstableFeatureDescription] = &[
     help_text: "Enable unstable Temporal API",
     show_in_help: true,
     kind: UnstableFeatureKind::Runtime,
-    config_option: ConfigFileOption::SameAsFlagName,
     env_var: None,
   },
   UnstableFeatureDescription {
@@ -197,7 +169,6 @@ pub static FEATURE_DESCRIPTIONS: &[UnstableFeatureDescription] = &[
     help_text: "Enable unsafe __proto__ support. This is a security risk.",
     show_in_help: true,
     kind: UnstableFeatureKind::Runtime,
-    config_option: ConfigFileOption::SameAsFlagName,
     env_var: None,
   },
   UnstableFeatureDescription {
@@ -205,7 +176,6 @@ pub static FEATURE_DESCRIPTIONS: &[UnstableFeatureDescription] = &[
     help_text: "Enable unstable VSOCK APIs",
     show_in_help: false,
     kind: UnstableFeatureKind::Runtime,
-    config_option: ConfigFileOption::SameAsFlagName,
     env_var: None,
   },
   UnstableFeatureDescription {
@@ -213,7 +183,6 @@ pub static FEATURE_DESCRIPTIONS: &[UnstableFeatureDescription] = &[
     help_text: "Enable unstable WebGPU APIs",
     show_in_help: true,
     kind: UnstableFeatureKind::Runtime,
-    config_option: ConfigFileOption::SameAsFlagName,
     env_var: None,
   },
   UnstableFeatureDescription {
@@ -221,7 +190,6 @@ pub static FEATURE_DESCRIPTIONS: &[UnstableFeatureDescription] = &[
     help_text: "Enable unstable Web Worker APIs",
     show_in_help: true,
     kind: UnstableFeatureKind::Runtime,
-    config_option: ConfigFileOption::SameAsFlagName,
     env_var: None,
   },
   UnstableFeatureDescription {
@@ -229,7 +197,6 @@ pub static FEATURE_DESCRIPTIONS: &[UnstableFeatureDescription] = &[
     help_text: "Enable unstable bundle runtime API",
     show_in_help: true,
     kind: UnstableFeatureKind::Runtime,
-    config_option: ConfigFileOption::SameAsFlagName,
     env_var: None,
   },
   UnstableFeatureDescription {
@@ -237,7 +204,6 @@ pub static FEATURE_DESCRIPTIONS: &[UnstableFeatureDescription] = &[
     help_text: "Enable unstable TypeScript Go integration",
     show_in_help: true,
     kind: UnstableFeatureKind::Cli,
-    config_option: ConfigFileOption::SameAsFlagName,
     env_var: Some("DENO_UNSTABLE_TSGO"),
   },
 ];
