@@ -35,7 +35,7 @@ use deno_core::error::CoreError;
 use deno_core::error::JsError;
 use deno_core::merge_op_metrics;
 use deno_core::v8;
-use deno_cron::local::LocalCronHandler;
+use deno_cron::CronHandlerImpl;
 use deno_fs::FileSystem;
 use deno_io::Stdio;
 use deno_kv::dynamic::MultiBackendDbHandler;
@@ -1054,7 +1054,7 @@ fn common_extensions<
     deno_net::deno_net::lazy_init(),
     deno_tls::deno_tls::init(),
     deno_kv::deno_kv::lazy_init::<MultiBackendDbHandler>(),
-    deno_cron::deno_cron::init(LocalCronHandler::new()),
+    deno_cron::deno_cron::init(CronHandlerImpl::create_from_env()),
     deno_napi::deno_napi::lazy_init(),
     deno_http::deno_http::lazy_init(),
     deno_io::deno_io::lazy_init(),
@@ -1129,7 +1129,6 @@ fn common_runtime(opts: CommonRuntimeOptions) -> JsRuntime {
     validate_import_attributes_cb: Some(
       create_validate_import_attributes_callback(enable_raw_imports.clone()),
     ),
-    import_assertions_support: deno_core::ImportAssertionsSupport::Error,
     maybe_op_stack_trace_callback: opts
       .enable_stack_trace_arg_in_ops
       .then(create_permissions_stack_trace_callback),
