@@ -1,15 +1,13 @@
-// Copyright 2018-2025 the Deno authors. MIT license.
+// Copyright 2018-2026 the Deno authors. MIT license.
 
 use test_util as util;
+use test_util::eprintln;
+use test_util::test;
 use util::TestContext;
 use util::TestContextBuilder;
 use util::assert_not_contains;
 use util::testdata_path;
 
-#[cfg_attr(
-  all(target_os = "macos", target_arch = "x86_64", debug_assertions),
-  ignore // TODO: fix this, see https://github.com/denoland/sui/issues/43
-)]
 #[test]
 fn compile_basic() {
   let context = TestContextBuilder::new().build();
@@ -61,10 +59,6 @@ fn compile_basic() {
   output.assert_matches_text("Welcome to Deno!\n");
 }
 
-#[cfg_attr(
-  all(target_os = "macos", target_arch = "x86_64", debug_assertions),
-  ignore // TODO: fix this, see https://github.com/denoland/sui/issues/43
-)]
 #[test]
 fn standalone_args() {
   let context = TestContextBuilder::new().build();
@@ -96,10 +90,6 @@ fn standalone_args() {
     .assert_matches_text("a\nb\nfoo\n--bar\n--unstable\n");
 }
 
-#[cfg_attr(
-  all(target_os = "macos", target_arch = "x86_64", debug_assertions),
-  ignore // TODO: fix this, see https://github.com/denoland/sui/issues/43
-)]
 #[test]
 fn standalone_load_datauri() {
   let context = TestContextBuilder::new().build();
@@ -129,10 +119,6 @@ fn standalone_load_datauri() {
 }
 
 // https://github.com/denoland/deno/issues/13704
-#[cfg_attr(
-  all(target_os = "macos", target_arch = "x86_64", debug_assertions),
-  ignore // TODO: fix this, see https://github.com/denoland/sui/issues/43
-)]
 #[test]
 fn standalone_follow_redirects() {
   let context = TestContextBuilder::new().build();
@@ -278,10 +264,6 @@ fn compile_and_overwrite_file() {
   }
 }
 
-#[cfg_attr(
-  all(target_os = "macos", target_arch = "x86_64", debug_assertions),
-  ignore // TODO: fix this, see https://github.com/denoland/sui/issues/43
-)]
 #[test]
 fn standalone_runtime_flags() {
   let context = TestContextBuilder::new().build();
@@ -312,16 +294,12 @@ fn standalone_runtime_flags() {
     .split_output()
     .run()
     .assert_exit_code(1)
-    .assert_stdout_matches_text("0.1472050634010581\n")
+    .assert_stdout_matches_text("0.34158532233056826\n")
     .assert_stderr_matches_text(
       "[WILDCARD]NotCapable: Requires write access to[WILDCARD]",
     );
 }
 
-#[cfg_attr(
-  all(target_os = "macos", target_arch = "x86_64", debug_assertions),
-  ignore // TODO: fix this, see https://github.com/denoland/sui/issues/43
-)]
 #[test]
 fn standalone_ext_flag_ts() {
   let context = TestContextBuilder::new().build();
@@ -353,10 +331,6 @@ fn standalone_ext_flag_ts() {
     .assert_matches_text("executing typescript with no extension\n");
 }
 
-#[cfg_attr(
-  all(target_os = "macos", target_arch = "x86_64", debug_assertions),
-  ignore // TODO: fix this, see https://github.com/denoland/sui/issues/43
-)]
 #[test]
 fn standalone_ext_flag_js() {
   let context = TestContextBuilder::new().build();
@@ -387,10 +361,6 @@ fn standalone_ext_flag_js() {
     .assert_matches_text("executing javascript with no extension\n");
 }
 
-#[cfg_attr(
-  all(target_os = "macos", target_arch = "x86_64", debug_assertions),
-  ignore // TODO: fix this, see https://github.com/denoland/sui/issues/43
-)]
 #[test]
 fn standalone_import_map() {
   let context = TestContextBuilder::new().build();
@@ -422,10 +392,6 @@ fn standalone_import_map() {
     .assert_exit_code(0);
 }
 
-#[cfg_attr(
-  all(target_os = "macos", target_arch = "x86_64", debug_assertions),
-  ignore // TODO: fix this, see https://github.com/denoland/sui/issues/43
-)]
 #[test]
 fn standalone_import_map_config_file() {
   let context = TestContextBuilder::new().build();
@@ -457,10 +423,6 @@ fn standalone_import_map_config_file() {
     .assert_exit_code(0);
 }
 
-#[cfg_attr(
-  all(target_os = "macos", target_arch = "x86_64", debug_assertions),
-  ignore // TODO: fix this, see https://github.com/denoland/sui/issues/43
-)]
 #[test]
 // https://github.com/denoland/deno/issues/12670
 fn skip_rebundle() {
@@ -540,10 +502,6 @@ fn check_local_by_default2() {
     );
 }
 
-#[cfg_attr(
-  all(target_os = "macos", target_arch = "x86_64", debug_assertions),
-  ignore // TODO: fix this, see https://github.com/denoland/sui/issues/43
-)]
 #[test]
 fn workers_basic() {
   let context = TestContext::with_http_server();
@@ -574,10 +532,6 @@ fn workers_basic() {
     .assert_matches_file("./compile/workers/basic.out");
 }
 
-#[cfg_attr(
-  all(target_os = "macos", target_arch = "x86_64", debug_assertions),
-  ignore // TODO: fix this, see https://github.com/denoland/sui/issues/43
-)]
 #[test]
 fn workers_not_in_module_map() {
   let context = TestContext::with_http_server();
@@ -607,10 +561,6 @@ fn workers_not_in_module_map() {
   ));
 }
 
-#[cfg_attr(
-  all(target_os = "macos", target_arch = "x86_64", debug_assertions),
-  ignore // TODO: fix this, see https://github.com/denoland/sui/issues/43
-)]
 #[test]
 fn workers_with_include_flag() {
   let context = TestContext::with_http_server();
@@ -642,10 +592,6 @@ fn workers_with_include_flag() {
     .assert_matches_text("Hello from worker!\nReceived 42\nClosing\n");
 }
 
-#[cfg_attr(
-  all(target_os = "macos", target_arch = "x86_64", debug_assertions),
-  ignore // TODO: fix this, see https://github.com/denoland/sui/issues/43
-)]
 #[test]
 fn dynamic_import() {
   let context = TestContext::with_http_server();
@@ -676,10 +622,6 @@ fn dynamic_import() {
     .assert_exit_code(0);
 }
 
-#[cfg_attr(
-  all(target_os = "macos", target_arch = "x86_64", debug_assertions),
-  ignore // TODO: fix this, see https://github.com/denoland/sui/issues/43
-)]
 #[test]
 fn dynamic_import_unanalyzable() {
   let context = TestContext::with_http_server();
@@ -823,7 +765,9 @@ testing[WILDCARD]this
     .args("compile --output binary main.ts")
     .run()
     .assert_exit_code(0)
-    .assert_matches_text("Check file:///[WILDLINE]/main.ts\nCompile file:///[WILDLINE]/main.ts to binary[WILDLINE]\n");
+    .assert_matches_text(
+      "Check main.ts\nCompile main.ts to binary[WILDLINE]\n",
+    );
 
   context
     .new_command()
@@ -832,10 +776,6 @@ testing[WILDCARD]this
     .assert_matches_text("2\n");
 }
 
-#[cfg_attr(
-  all(target_os = "macos", target_arch = "x86_64", debug_assertions),
-  ignore // TODO: fix this, see https://github.com/denoland/sui/issues/43
-)]
 #[test]
 fn compile_npm_bin_esm() {
   run_npm_bin_compile_test(RunNpmBinCompileOptions {
@@ -852,10 +792,6 @@ fn compile_npm_bin_esm() {
 }
 
 #[test]
-#[cfg_attr(
-  all(target_os = "macos", target_arch = "x86_64", debug_assertions),
-  ignore // TODO: fix this, see https://github.com/denoland/sui/issues/43
-)]
 fn compile_npm_bin_cjs() {
   run_npm_bin_compile_test(RunNpmBinCompileOptions {
     input_specifier: "npm:@denotest/bin/cli-cjs",
@@ -871,10 +807,6 @@ fn compile_npm_bin_cjs() {
 }
 
 #[test]
-#[cfg_attr(
-  all(target_os = "macos", target_arch = "x86_64", debug_assertions),
-  ignore // TODO: fix this, see https://github.com/denoland/sui/issues/43
-)]
 fn compile_npm_cowsay_main() {
   run_npm_bin_compile_test(RunNpmBinCompileOptions {
     input_specifier: "npm:cowsay@1.5.0",
@@ -890,10 +822,6 @@ fn compile_npm_cowsay_main() {
 }
 
 #[test]
-#[cfg_attr(
-  all(target_os = "macos", target_arch = "x86_64", debug_assertions),
-  ignore // TODO: fix this, see https://github.com/denoland/sui/issues/43
-)]
 fn compile_npm_no_permissions() {
   run_npm_bin_compile_test(RunNpmBinCompileOptions {
     input_specifier: "npm:@denotest/cli-with-permissions@1.0.0",
@@ -909,10 +837,6 @@ fn compile_npm_no_permissions() {
 }
 
 #[test]
-#[cfg_attr(
-  all(target_os = "macos", target_arch = "x86_64", debug_assertions),
-  ignore // TODO: fix this, see https://github.com/denoland/sui/issues/43
-)]
 fn compile_npm_cowsay_explicit() {
   run_npm_bin_compile_test(RunNpmBinCompileOptions {
     input_specifier: "npm:cowsay@1.5.0/cowsay",
@@ -928,10 +852,6 @@ fn compile_npm_cowsay_explicit() {
 }
 
 #[test]
-#[cfg_attr(
-  all(target_os = "macos", target_arch = "x86_64", debug_assertions),
-  ignore // TODO: fix this, see https://github.com/denoland/sui/issues/43
-)]
 fn compile_npm_cowthink() {
   run_npm_bin_compile_test(RunNpmBinCompileOptions {
     input_specifier: "npm:cowsay@1.5.0/cowthink",
@@ -1005,10 +925,6 @@ fn run_npm_bin_compile_test(opts: RunNpmBinCompileOptions) {
   output.assert_matches_file(opts.output_file);
 }
 
-#[cfg_attr(
-  all(target_os = "macos", target_arch = "x86_64", debug_assertions),
-  ignore // TODO: fix this, see https://github.com/denoland/sui/issues/43
-)]
 #[test]
 fn compile_node_modules_symlink_outside() {
   // this code is using a canonicalized temp dir because otherwise
@@ -1077,10 +993,6 @@ fn compile_node_modules_symlink_outside() {
   output.assert_matches_file("compile/node_modules_symlink_outside/main.out");
 }
 
-#[cfg_attr(
-  all(target_os = "macos", target_arch = "x86_64", debug_assertions),
-  ignore // TODO: fix this, see https://github.com/denoland/sui/issues/43
-)]
 #[test]
 fn compile_node_modules_symlink_non_existent() {
   let context = TestContextBuilder::for_npm().use_temp_cwd().build();
@@ -1104,8 +1016,8 @@ console.log(getValue());"#,
     r#"Download http://localhost:4260/@denotest%2fesm-basic
 Download http://localhost:4260/@denotest/esm-basic/1.0.0.tgz
 Initialize @denotest/esm-basic@1.0.0
-Check file:///[WILDCARD]/main.ts
-Compile file:///[WILDCARD]/main.ts to [WILDCARD]
+Check main.ts
+Compile main.ts to [WILDCARD]
 Warning Failed resolving symlink. Ignoring.
     Path: [WILDCARD]
     Message: [WILDCARD])
@@ -1125,10 +1037,6 @@ Embedded Files
 }
 
 #[test]
-#[cfg_attr(
-  all(target_os = "macos", target_arch = "x86_64", debug_assertions),
-  ignore // TODO: fix this, see https://github.com/denoland/sui/issues/43
-)]
 fn dynamic_imports_tmp_lit() {
   let context = TestContextBuilder::new().build();
   let dir = context.temp_dir();
@@ -1153,10 +1061,6 @@ fn dynamic_imports_tmp_lit() {
 }
 
 #[test]
-#[cfg_attr(
-  all(target_os = "macos", target_arch = "x86_64", debug_assertions),
-  ignore // TODO: fix this, see https://github.com/denoland/sui/issues/43
-)]
 fn granular_unstable_features() {
   let context = TestContextBuilder::new().build();
   let dir = context.temp_dir();
@@ -1184,10 +1088,6 @@ fn granular_unstable_features() {
 }
 
 #[test]
-#[cfg_attr(
-  all(target_os = "macos", target_arch = "x86_64", debug_assertions),
-  ignore // TODO: fix this, see https://github.com/denoland/sui/issues/43
-)]
 fn granular_unstable_features_config_file() {
   let context = TestContextBuilder::new().use_temp_cwd().build();
   let dir = context.temp_dir();
@@ -1225,10 +1125,6 @@ fn granular_unstable_features_config_file() {
 }
 
 #[test]
-#[cfg_attr(
-  all(target_os = "macos", target_arch = "x86_64", debug_assertions),
-  ignore // TODO: fix this, see https://github.com/denoland/sui/issues/43
-)]
 fn dynamic_import_bad_data_uri() {
   let context = TestContextBuilder::new().build();
   let dir = context.temp_dir();
@@ -1258,10 +1154,6 @@ fn dynamic_import_bad_data_uri() {
 }
 
 #[test]
-#[cfg_attr(
-  all(target_os = "macos", target_arch = "x86_64", debug_assertions),
-  ignore // TODO: fix this, see https://github.com/denoland/sui/issues/43
-)]
 fn standalone_config_file_respects_compiler_options() {
   let context = TestContextBuilder::new().build();
   let dir = context.temp_dir();
@@ -1291,10 +1183,6 @@ fn standalone_config_file_respects_compiler_options() {
 }
 
 #[test]
-#[cfg_attr(
-  all(target_os = "macos", target_arch = "x86_64", debug_assertions),
-  ignore // TODO: fix this, see https://github.com/denoland/sui/issues/43
-)]
 fn standalone_jsr_dynamic_import() {
   let context = TestContextBuilder::for_jsr().build();
   let dir = context.temp_dir();
