@@ -1,4 +1,4 @@
-// Copyright 2018-2025 the Deno authors. MIT license.
+// Copyright 2018-2026 the Deno authors. MIT license.
 
 use std::process::Output;
 use std::sync::Arc;
@@ -16,6 +16,8 @@ use test_util::DenoChild;
 use test_util::TestContext;
 use test_util::TestContextBuilder;
 use test_util::assertions::assert_json_subset;
+use test_util::eprintln;
+use test_util::test;
 use tokio::sync::Mutex;
 use tokio::time::timeout;
 use uuid::Uuid;
@@ -486,7 +488,7 @@ async fn setup() -> (TestContext, JupyterClient, JupyterServerProcess) {
   (context, client, process)
 }
 
-#[tokio::test]
+#[test]
 async fn jupyter_heartbeat_echoes() -> Result<()> {
   let (_ctx, client, _process) = setup().await;
   client.send_heartbeat(b"ping").await?;
@@ -496,7 +498,7 @@ async fn jupyter_heartbeat_echoes() -> Result<()> {
   Ok(())
 }
 
-#[tokio::test]
+#[test]
 async fn jupyter_kernel_info() -> Result<()> {
   let (_ctx, client, _process) = setup().await;
   client
@@ -522,7 +524,7 @@ async fn jupyter_kernel_info() -> Result<()> {
   Ok(())
 }
 
-#[tokio::test]
+#[test]
 async fn jupyter_execute_request() -> Result<()> {
   let (_ctx, client, _process) = setup().await;
   let request = client
@@ -599,7 +601,7 @@ async fn jupyter_execute_request() -> Result<()> {
   Ok(())
 }
 
-#[tokio::test]
+#[test]
 async fn jupyter_store_history_false() -> Result<()> {
   let (_ctx, client, _process) = setup().await;
   client
@@ -627,7 +629,7 @@ async fn jupyter_store_history_false() -> Result<()> {
   Ok(())
 }
 
-#[tokio::test]
+#[test]
 async fn jupyter_http_server() -> Result<()> {
   let (_ctx, client, _process) = setup().await;
   client
