@@ -13,3 +13,9 @@ Deno.test("napi string", function () {
   assertEquals(strings.test_utf16(""), "");
   assertEquals(strings.test_utf16("🦕"), "🦕");
 });
+
+Deno.test("napi string utf8 roundtrip", function () {
+  assertEquals(strings.test_utf8_roundtrip(""), "");
+  assertEquals(strings.test_utf8_roundtrip("hello"), "hello");
+  assertEquals(strings.test_utf8_roundtrip("🦕"), "🦕");
+});
